# -*- coding: utf-8 -*-
from .pymysql import connect,escape_string
# import config.conf as config
import kcweb.config as config
import time,traceback,decimal,random,copy,hashlib
from kcweb.utill.cache import cache as kcwcache
mycache=copy.deepcopy(config.cache)
mycache['type']='Python'

dbconfig=config.database
def md5(strs):
    """md5加密"""
    if not strs:
        return strs
    m = hashlib.md5()
    b = strs.encode(encoding='utf-8')
    m.update(b)
    return m.hexdigest()
class mysql:
    """数据库实例"""
    __config=dbconfig
    __conn={} #数据库链接对象
    __connlists=[] #短连接列表  用于关闭
    __cursor=None #游标对象
    __errorcount=dbconfig['break'] #允许最大链接错误次数
    __errorcounts=0 #默认链接错误次数 
    __dbObjcount=dbconfig['dbObjcount'] #数据库链接实例数量
    __sql=''
    __sqls=''
    __masteridentifier='' # 主服务器标识
    __slaveidentifier='' # 从服务器标识
    def __del__(self):
        pass
    def close(self):
        "关闭连接,web一下模式下该方法无需理会，框架自动完成"
        if not self.__config['pattern'] and mysql.__conn:
            if self.__connlists:
                for k in self.__connlists:
                    try:
                        mysql.__conn[k].close()
                    except Exception as e:
                        if self.__config['debug']:
                            print("mysql短连接关闭失败",str(e),k)
                    else:
                        if self.__config['debug']:
                            print("mysql短连接关闭成功",k)
                mysql.__connlists=[]
                mysql.__conn={}
            elif isinstance(mysql.__conn,dict):
                i=0
                for thost in self.__config['host']:
                    identifier=thost+str(self.__config['port'][i])+self.__config['user'][i]+self.__config['password'][i]+self.__config['db'][i]
                    # print(mysql.__conn)
                    for k in mysql.__conn[identifier]:
                        try:
                            k['obj'].close()
                            if self.__config['debug']:
                                print("mysql短连接已关闭",k)
                        except Exception as e:
                            if self.__config['debug']:
                                print("mysql短连接关闭失败",str(e),k)
                    i+=1
                mysql.__conn={}
                self.__connlists=[]
            elif isinstance(mysql.__conn,object):
                mysql.__conn[self.__masteridentifier].close()
                mysql.__conn={}
                if self.__config['debug']:
                    print("mysql短连接已关闭",mysql.__conn)
        mysql.__config=copy.deepcopy(dbconfig)
    __dbcount=1
    def __setdbcount(self):
        "设置数据库配置总数量"
        if isinstance(self.__config['host'],str):
            self.__config['host']=[self.__config['host']]
        if isinstance(self.__config['port'],str):
            self.__config['port']=[self.__config['port']]
        if isinstance(self.__config['user'],str):
            self.__config['user']=[self.__config['user']]
        if isinstance(self.__config['password'],str):
            self.__config['password']=[self.__config['password']]
        if isinstance(self.__config['db'],str):
            self.__config['db']=[self.__config['db']]
        host=len(self.__config['host'])
        port=len(self.__config['port'])
        user=len(self.__config['user'])
        password=len(self.__config['password'])
        db=len(self.__config['db'])
        lists=[host,port,user,password,db]
        lists.sort()
        self.__dbcount=lists[0]
    def __closeconn(self,identifier):
        "长链接模式下，关闭链接池的链接"
        if self.__config['pattern']:
            if identifier in mysql.__conn:
                for k in mysql.__conn[identifier]:
                    try:
                        k['obj'].close()
                        if self.__config['debug']:
                            print("mysql长连接关闭成功",k)
                    except Exception as e:
                        if self.__config['debug']:
                            print(k,"mysql长连接关闭失败",str(e))
                mysql.__conn[identifier]=[]
    __dbobjident=None #集中式(单一服务器)并且长连接模式下随机服务器链接标识  和 分布式(主从服务器)模式下随机服务器链接标识
    def __connects(self,typess="DQL"):
        """设置数据库链接
        
        参数 typess ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
        """
        try:
            if self.__config['deploy']==0: # 集中式(单一服务器)
                if self.__config['pattern']: # 长连接情况下
                    self.__masteridentifier=self.__config['host'][0]+str(self.__config['port'][0])+self.__config['user'][0]+self.__config['password'][0]+self.__config['db'][0] # 服务器标识
                    if self.__masteridentifier not in mysql.__conn or len(mysql.__conn[self.__masteridentifier])<1:
                        i=0
                        masterlistsdb=[]
                        while i<self.__dbObjcount: #创建self.__dbObjcount个数据库链接实例
                            obj=connect(host=self.__config['host'][0], port=self.__config['port'][0], user=self.__config['user'][0], password=self.__config['password'][0], db=self.__config['db'][0], charset=self.__config['charset'])
                            objar={"obj":obj,"error":0}
                            masterlistsdb.append(objar)
                            i=i+1
                        mysql.__conn[self.__masteridentifier]=masterlistsdb
                        if self.__config['debug']:
                            print("第%s次创建数据库链接对象，长连接模式" % (self.__errorcounts+1))
                else:
                    self.__masteridentifier=self.__config['host'][0]+str(self.__config['port'][0])+self.__config['user'][0]+self.__config['password'][0]+self.__config['db'][0] # 服务器标识
                    try:
                        mysql.__conn[self.__masteridentifier]
                    except KeyError: # 铺获未知异常
                        mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][0], port=self.__config['port'][0], user=self.__config['user'][0], password=self.__config['password'][0], db=self.__config['db'][0], charset=self.__config['charset'])
                        self.__connlists.append(self.__masteridentifier)
                        if self.__config['debug']:
                            print("mysql短连接已创建",self.__masteridentifier,mysql.__conn[self.__masteridentifier])
            elif self.__config['deploy']==1: # 分布式(主从服务器)
                if self.__config['pattern']: # 长连接情况下
                    j=0
                    self.__masteridentifier=''
                    while j < self.__config['master_num']:
                        self.__masteridentifier=self.__masteridentifier+self.__config['host'][j]+str(self.__config['port'][j])+self.__config['user'][j]+self.__config['password'][j]+self.__config['db'][j] # 主服务器标识
                        j=j+1
                    j=self.__config['master_num']
                    self.__slaveidentifier=''
                    while j < self.__dbcount:
                        self.__slaveidentifier=self.__slaveidentifier+self.__config['host'][j]+str(self.__config['port'][j])+self.__config['user'][j]+self.__config['password'][j]+self.__config['db'][j] # 从服务器标识
                        j=j+1
                    if self.__masteridentifier not in mysql.__conn or len(mysql.__conn[self.__masteridentifier])<self.__config['master_num']:
                        j=0
                        masterlistsdb=[] #主服务器实例
                        while j < self.__config['master_num']: #主服务器数量
                            i=0
                            while i<self.__dbObjcount: #创建self.__dbObjcount个数据库链接实例
                                obj=connect(host=self.__config['host'][j], port=self.__config['port'][j], user=self.__config['user'][j], password=self.__config['password'][j], db=self.__config['db'][j], charset=self.__config['charset'])
                                objar={"obj":obj,"error":0}
                                masterlistsdb.append(objar)
                                i=i+1
                            j=j+1
                        mysql.__conn[self.__masteridentifier]=masterlistsdb
                        if self.__config['debug']:
                            print("%d次创建数据库链接对象，长连接模式（主）" % (self.__errorcounts+1))
                    if self.__slaveidentifier not in mysql.__conn or len(mysql.__conn[self.__slaveidentifier])<self.__dbcount-self.__config['master_num']:
                        
                        j=self.__config['master_num']
                        slaveerlistsdb=[] #从服务器实例
                        while j < self.__dbcount: #从服务器数量
                            i=0
                            while i<self.__dbObjcount: #创建self.__dbObjcount个数据库链接实例
                                obj=connect(host=self.__config['host'][j], port=self.__config['port'][j], user=self.__config['user'][j], password=self.__config['password'][j], db=self.__config['db'][j], charset=self.__config['charset'])
                                objar={"obj":obj,"error":0}
                                slaveerlistsdb.append(objar)
                                i=i+1
                            j=j+1
                        mysql.__conn[self.__slaveidentifier]=slaveerlistsdb
                        if self.__config['debug']:
                            print("%d创建数据库链接对象，长连接模式（从）" % (self.__errorcounts+1))
                else:
                    if typess == "DQL": #数据查询语言DQL
                        if self.__config['master_dql']: #所有服务器随机
                            self.__dbobjident=random.randint(0,self.__dbcount-1)
                            self.__masteridentifier=self.__config['host'][self.__dbobjident]+str(self.__config['port'][self.__dbobjident])+self.__config['user'][self.__dbobjident]+self.__config['password'][self.__dbobjident]+self.__config['db'][self.__dbobjident] # 服务器标识
                            try:
                                mysql.__conn[self.__masteridentifier]
                            except KeyError:
                                mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][self.__dbobjident], 
                                    port=self.__config['port'][self.__dbobjident], 
                                    user=self.__config['user'][self.__dbobjident], 
                                    password=self.__config['password'][self.__dbobjident], 
                                    db=self.__config['db'][self.__dbobjident], 
                                    charset=self.__config['charset']
                                )
                                self.__connlists.append(self.__masteridentifier)
                                if self.__config['debug']:
                                    print("创建所有数据库链接对象",(self.__errorcounts+1),self.__config['host'][self.__dbobjident])
                        else: #从服务器随机
                            self.__dbobjident=random.randint(self.__config['master_num'],self.__dbcount-1)
                            self.__masteridentifier=self.__config['host'][self.__dbobjident]+str(self.__config['port'][self.__dbobjident])+self.__config['user'][self.__dbobjident]+self.__config['password'][self.__dbobjident]+self.__config['db'][self.__dbobjident] # 服务器标识
                            try:
                                mysql.__conn[self.__masteridentifier]
                            except KeyError:
                                mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][self.__dbobjident], 
                                    port=self.__config['port'][self.__dbobjident], 
                                    user=self.__config['user'][self.__dbobjident], 
                                    password=self.__config['password'][self.__dbobjident], 
                                    db=self.__config['db'][self.__dbobjident], 
                                    charset=self.__config['charset']
                                )
                                self.__connlists.append(self.__masteridentifier)
                                if self.__config['debug']:
                                    print("创建从数据库链接对象",str(self.__errorcounts+1),self.__config['host'][self.__dbobjident],str(self.__config['db'][self.__dbobjident]))
                    else:
                         #从服务器随机
                        self.__dbobjident=random.randint(self.__config['master_num'],self.__dbcount-1)
                        self.__masteridentifier=self.__config['host'][self.__dbobjident]+str(self.__config['port'][self.__dbobjident])+self.__config['user'][self.__dbobjident]+self.__config['password'][self.__dbobjident]+self.__config['db'][self.__dbobjident] # 服务器标识
                        try:
                            mysql.__conn[self.__masteridentifier]
                        except KeyError: 
                            mysql.__conn[self.__masteridentifier]=connect(host=self.__config['host'][self.__dbobjident], 
                                port=self.__config['port'][self.__dbobjident], 
                                user=self.__config['user'][self.__dbobjident], 
                                password=self.__config['password'][self.__dbobjident], 
                                db=self.__config['db'][self.__dbobjident], 
                                charset=self.__config['charset']
                            )
                            self.__connlists.append(self.__masteridentifier)
                            if self.__config['debug']:
                                print("创建从数据库链接对象" , (self.__errorcounts+1),self.__config['host'][self.__dbobjident],str(self.__config['db'][self.__dbobjident]))
                    
        except Exception as e: # 铺获未知异常
            try:
                errcome=tuple(eval(str(e)))[0]
            except Exception as e: # 铺获未知异常
                raise Exception(e)
            if errcome == 2003: #如果数据库链接失败
                # self.__closeconn(self.__masteridentifier)
                # self.__closeconn(self.__slaveidentifier)
                if self.__config['cli']:
                    time.sleep(10)
                    self.__errorcounts=self.__errorcounts+1
                    if self.__errorcounts<self.__errorcount*300:
                        if self.__config['debug']:
                            print("无法链接到数据库服务器，准备第%d次重新链接:%s" % (self.__errorcounts,e))
                        self.__connects(typess)
                    else:
                        # self.__config=copy.deepcopy(dbconfig)
                        self.__errorcounts=0
                        raise Exception(e)
                else:
                    self.__errorcounts=self.__errorcounts+1
                    if self.__errorcounts<self.__errorcount:
                        if self.__config['debug']:
                            print("无法链接到数据库服务器，开始重新链接")
                        self.__connects(typess)
                    else:
                        # self.__config=copy.deepcopy(dbconfig)
                        self.__errorcounts=0
                        raise Exception(e)
            else:
                raise Exception(e)
        else:
            self.__errorcounts=0

    def connect(self,config):
        """设置数据库链接信息 

        参数 config 参考配置信息格式  可以设置数据库名（以字符串形式）

        返回 mysql对象
        """ 
        if config:
            if isinstance(config,dict):
                if "type" in config:
                    self.__config['type']=config['type']
                if "host" in config:
                    self.__config['host']=config['host']
                if "port" in config:
                    self.__config['port']=config['port']
                if "user" in config:
                    self.__config['user']=config['user']
                if "password" in config:
                    self.__config['password']=config['password']
                if "db" in config:
                    self.__config['db']=config['db']
                if "charset" in config:
                    self.__config['charset']=config['charset']
                if "pattern" in config:
                    self.__config['pattern']=config['pattern']
                if "cli" in config:
                    self.__config['cli']=config['cli']
                if "dbObjcount" in config:
                    self.__config['dbObjcount']=config['dbObjcount']
                if "deploy" in config:
                    self.__config['deploy']=config['deploy']
                if "master_num" in config:
                    self.__config['master_num']=config['master_num']
                if "master_dql" in config:
                    self.__config['master_dql']=config['master_dql']
                if "break" in config:
                    self.__config['break']=config['break']
            elif isinstance(config,str):
                self.__config['db']=[]
                i=0
                if not self.__dbcount:
                    self.__dbcount=len(self.__config['host'])
                while i<self.__dbcount:
                    self.__config['db'].append(config)
                    i=i+1
            else:
                print("config类型错误，设置连接不生效")
        return self
    __table=""
    def table(self,table):
        """设置表名

        参数 table：str 表名
        """
        self.__table=table
        return self
    __patternerrorcount=0 # 长连接情况下 错误次数
    def __setcursor(self,typess='DQL'):
        """设置游标

        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
        """
        self.__setdbcount()
        # self.__DBsingle=self.__config['pattern']
        self.__dbObjcount=self.__config['dbObjcount']
        self.__errorcount=self.__config['break']
        self.__connects()
        bs=''
        try:
            if self.__config['deploy']==0: # 集中式(单一服务器)
                if self.__config['pattern']: # 长连接情况下
                    bs=self.__masteridentifier
                    # types='master'
                    self.__dbobjident=random.randint(0, self.__dbObjcount-1)
                    self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例
                else:
                    self.__cursor=self.__conn[self.__masteridentifier].cursor() #获取随机数据库链接实例
            elif self.__config['deploy']==1: # 分布式(主从服务器)
                if self.__config['pattern']: # 长连接情况下
                    bs=self.__masteridentifier
                    slavecount=self.__dbcount-self.__config['master_num'] #从服务器数量
                    # types='master'
                    if typess=='DQL': #数据查询语言DQL
                        if self.__config['master_dql']:  #主服务器可以执行DQL
                            if slavecount:
                                if random.randint(0,1):
                                    bs=self.__masteridentifier
                                    # types='master'
                                    self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
                                    self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
                                else:
                                    bs=self.__slaveidentifier
                                    # types='slave'
                                    self.__dbobjident=random.randint(0,self.__dbObjcount*slavecount-1)
                                    self.__cursor=self.__conn[self.__slaveidentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 从服务器
                            else:
                                bs=self.__masteridentifier
                                # types='master'
                                self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
                                self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
                        else:
                            bs=self.__slaveidentifier
                            # types='slave'
                            self.__dbobjident=random.randint(0,self.__dbObjcount*slavecount-1)
                            self.__cursor=self.__conn[self.__slaveidentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 从服务器
                    else: #数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
                        bs=self.__masteridentifier
                        # types='master'
                        self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
                        self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
                    self.__conn[bs][self.__dbobjident]['error']=0
                else:
                    self.__cursor=self.__conn[self.__masteridentifier].cursor()
        except Exception as e: # 铺获未知异常
            raise Exception(e)
        else:
            self.__patternerrorcount=0
            return self.__cursor
    
    def get_exec_cursor(self):
        """获取执行游标（不推荐使用）
        
        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
        """
        cursor=self.__setcursor("DML")
        return cursor
    def get_query_cursor(self):
        """获取查询游标（不推荐使用）
        
        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
        """
        cursor=self.__setcursor("DQL")
        return cursor
    def __execute(self,typess='DQL'):
        """执行sql语句
        
        参数 type ：数据查询语言DQL，数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
        """
        self.__setdbcount()
        # self.__DBsingle=self.__config['pattern']
        self.__dbObjcount=self.__config['dbObjcount']
        self.__errorcount=self.__config['break']
        self.__connects()
        try:
            if self.__config['deploy']==0: # 集中式(单一服务器)
                if self.__config['pattern']: # 长连接情况下
                    bs=self.__masteridentifier
                    types='master'
                    self.__dbobjident=random.randint(0, self.__dbObjcount-1)
                    self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例
                    res=self.__cursor.execute(self.__sql)
                else:
                    self.__cursor=self.__conn[self.__masteridentifier].cursor() #获取随机数据库链接实例
                    res=self.__cursor.execute(self.__sql)
            elif self.__config['deploy']==1: # 分布式(主从服务器)
                if self.__config['pattern']: # 长连接情况下
                    bs=self.__masteridentifier
                    slavecount=self.__dbcount-self.__config['master_num'] #从服务器数量
                    types='master'
                    if typess=='DQL': #数据查询语言DQL
                        if self.__config['master_dql']:  #主服务器可以执行DQL
                            if slavecount:
                                if random.randint(0,1):
                                    bs=self.__masteridentifier
                                    types='master'
                                    self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
                                    self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
                                else:
                                    bs=self.__slaveidentifier
                                    types='slave'
                                    self.__dbobjident=random.randint(0,self.__dbObjcount*slavecount-1)
                                    self.__cursor=self.__conn[self.__slaveidentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 从服务器
                            else:
                                bs=self.__masteridentifier
                                types='master'
                                self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
                                self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
                        else:
                            bs=self.__slaveidentifier
                            types='slave'
                            self.__dbobjident=random.randint(0,self.__dbObjcount*slavecount-1)
                            self.__cursor=self.__conn[self.__slaveidentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 从服务器
                    else: #数据操纵语言DML，数据定义语言DDL，数据控制语言DCL
                        bs=self.__masteridentifier
                        types='master'
                        self.__dbobjident=random.randint(0,self.__dbObjcount*self.__config['master_num']-1)
                        self.__cursor=self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].cursor() #获取随机数据库链接实例 主服务器
                    res=self.__cursor.execute(self.__sql)
                    self.__conn[bs][self.__dbobjident]['error']=0
                else:
                    self.__cursor=self.__conn[self.__masteridentifier].cursor()
                    res=self.__cursor.execute(self.__sql)
        except Exception as e: # 铺获未知异常
            try:
                errorcodes=tuple(eval(str(e)))[0]
            except:
                errorcodes=0
            if errorcodes == 2013 or errorcodes == 2006 or errorcodes == 2003 or errorcodes == 1053 or errorcodes == 0:  #2013连接丢失、2003数据库无法连接时、1053连接正在被关闭时、2006服务器丢失
                if self.__config['pattern']: # 长连接情况下
                    if self.__config['cli']==True:
                        if self.__config['debug']:
                            print("等待连接数据库")
                        if self.__patternerrorcount>self.__config['dbObjcount'] * self.__dbcount: #长连接情况下如果错误次数超过数据实例数量 关闭使用连接进行重连接
                            self.__patternerrorcount=0
                            if self.__config['deploy'] == 1: #分布式(主从服务器) 情况下
                                if self.__config['debug']:
                                    print("数据库连接失效，关闭主从连接池后重新连接")
                                self.__closeconn(self.__masteridentifier)
                                self.__closeconn(self.__slaveidentifier)
                                time.sleep(10)
                                self.__connects(typess)
                                self.__execute(typess)
                            else:
                                if self.__config['debug']:
                                    print("数据库连接失效，关闭主连接池后重新连接")
                                self.__closeconn(self.__masteridentifier)
                                time.sleep(10)
                                self.__connects(typess)
                                self.__execute(typess)
                        else:
                            self.__patternerrorcount=self.__patternerrorcount+1
                            self.__execute(typess)
                    else:
                        self.__conn[bs][self.__dbobjident]['error']=self.__conn[bs][self.__dbobjident]['error']+1 #当前数据库连接实例异常错误数量
                        if self.__conn[bs][self.__dbobjident]['error'] > 0:
                            try:
                                # mysql.__conn[bs][self.__dbobjident]['obj'].close() #关闭当前实例
                                self.__closeconn(bs)
                            except Exception as e:
                                if self.__config['debug']:
                                    print("关闭异常",e)
                            else:
                                self.__connects(typess)
                                self.__execute(typess)
                else: # 短连接情况下
                    if self.__config['debug']:
                        print("服务器正在被关闭，关闭当前连接后重试")
                    try:
                        del mysql.__conn[self.__masteridentifier]
                        mysql.__conn[self.__masteridentifier].close() #关闭当前实例
                    except Exception as e:
                        if self.__config['debug']:
                            print("关闭异常",e)
                    self.__connects(typess)
                    self.__execute(typess)
            else:
                raise Exception(e)
        else:
            self.__patternerrorcount=0
            return res

    def query(self,sql):
        """执行sql语句 注：只支持单一服务器模式

        参数 sql 字符串

        返回 列表  或  数字
        """
        self.__sql=sql
        res=self.__execute('DQL')
        description=self.__cursor.description #获取字段
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        self.__None()
        if description is None:
            return res
        else:
            lists=[]
            data_dict=[]
            for field in description:#获取字段
                data_dict.append(field[0])
            for k in result:
                i=0
                dicts={}
                for j in k:
                    dicts[data_dict[i]]=j
                    i=i+1
                lists.append(dicts)
            return lists
    def execute(self,sql):
        """执行sql语句 注：只支持单一服务器模式

        参数 sql 字符串

        返回 列表  或  数字
        """
        self.__sql=sql
        res=self.__execute('DML')
        description=self.__cursor.description #获取字段
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        self.__None()
        if description is None:
            return res
        else:
            lists=[]
            data_dict=[]
            for field in description:#获取字段
                data_dict.append(field[0])
            for k in result:
                i=0
                dicts={}
                for j in k:
                    dicts[data_dict[i]]=j
                    i=i+1
                lists.append(dicts)
            return lists
    def select(self,id=None):
        """select查询 

        返回 list(列表)
        """
        if id :
            self.__where="id=%d" % id
        self.__setsql()
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sqls
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        description=self.__cursor.description #获取字段
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        # self.__None()
        lists=[]
        keys =[]
        for field in description:#获取字段
            keys.append(field[0])
        key_number = len(keys)
        for row in result:
            item = dict()
            for q in range(key_number):
                k=row[q]
                if type(row[q])==decimal.Decimal:
                    k=float(row[q])
                item[keys[q]] = k
            lists.append(item)
        if lists and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),lists,self.__cache[1])
        self.__None()
        return lists
    def find(self,id=None):
        """查询一条记录
        
        返回 字典
        """
        if id :
            self.__where="id=%s" % id
        self.limit(1)
        self.__setsql()
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sqls
        if self.__cache and self.__cache[0]:
            item=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if item:
                self.__None()
                return item
        self.__execute()
        description=self.__cursor.description #获取字段
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        # self.__None()
        item = dict()
        keys =[]
        for field in description:#获取字段
            keys.append(field[0])
        key_number = len(keys)
        for row in result:
            for q in range(key_number):
                k=row[q]
                if type(row[q])==decimal.Decimal:
                    k=float(row[q])
                item[keys[q]] = k
        if item and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),item,self.__cache[1])
        self.__None()
        return item
    def value(self,field):
        """查询指定字段值
        
        返回 一个字段
        """
        self.__field=field
        self.limit(1)
        self.__setsql()
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sqls
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        # self.__None()
        strs=''
        if result:
            strs=result[0][0]
        if strs and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),strs,self.__cache[1])
        self.__None()
        return strs
    def count(self,field="*"):
        """查询数量
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('count')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sql
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        if self.__group:
            cou=len(result)
        else:
            try:
                cou=int(result[0][0])
            except IndexError:
                cou=0
        # self.__None()
        if cou and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
        self.__None()
        return cou
    def max(self,field):
        """查询某字段的最大值
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('max')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sql
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        cou=int(result[0][0])
        # self.__None()
        if cou and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
        self.__None()
        return cou
    def min(self,field):
        """查询某字段的最小值
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('min')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sql
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        # self.__None()
        cou=int(result[0][0])
        if cou and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
        self.__None()
        return cou
    def avg(self,field):
        """查询某字段的平均值
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('avg')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sql
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        # self.__None()
        cou=int(result[0][0])
        if cou and self.__cache and self.__cache[0]:
            kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
        self.__None()
        return cou
    def sum(self,field):
        """查询某字段之和
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('sum')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sql
        if self.__cache and self.__cache[0]:
            cou=kcwcache.cache().set_config(mycache).get_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])))
            if cou:
                self.__None()
                return cou
        self.__execute()
        result = self.__cursor.fetchall() #获取查询结果
        self.__cursor.close()
        # self.__None()
        cou=result[0][0]
        if not cou:
            cou=0
        else:
            if self.__cache and self.__cache[0]:
                kcwcache.cache().set_config(mycache).set_cache(md5(self.__sql+str(self.__config['db'][0])+str(self.__config['host'][0])),cou,self.__cache[1])
        self.__None()
        return cou

    def setinc(self,field,key=1,affair=False):
        """更新字段增加
         
        参数 field 要更新的字段

        参数 key 字段需要加多少

        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务
        """
        data={"field":field,"key":key}
        self.__setsql('setinc',data)
        res=self.__execute('DML')
        if affair==False and self.__startTrans==False:
            if not self.__config['pattern']:
                self.__conn[self.__masteridentifier].commit()
            else:
                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
        self.__cursor.close()
        self.__None()
        return res
    def update(self,data,affair=False):
        """数据表更新
         
        参数 data 要更新的内容  格式：{"name":"测试","age":20}

        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务
        """
        self.__setsql('update',data)
        res=self.__execute('DML')
        if affair==False and self.__startTrans==False:
            if not self.__config['pattern']:
                self.__conn[self.__masteridentifier].commit()
            else:
                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
        self.__cursor.close()
        self.__None()
        return res
    def delete(self,affair=False):
        """数据表删除

        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务
        """
        self.__setsql('delete')
        if self.__where:
            res=self.__execute('DML')
        else:
            return 0
        if affair==False and self.__startTrans==False:
            if not self.__config['pattern']:
                self.__conn[self.__masteridentifier].commit()
            else:
                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
        self.__cursor.close()
        self.__None()
        return res
    def insert(self,dicts,affair=False):
        """插入数据库 单条插入或多条插入

        参数 dicts 要插入的内容 单条格式：{"name":"测试","age":20}  。     多条格式：[{"name":"测试","age":20},{"name":"测试","age":20}]
        
        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务

        返回插入的数量
        """
        self.__setsql('insert',dicts)
        res=self.__execute('DML')
        if affair==False and self.__startTrans==False:
            if not self.__config['pattern']:
                self.__conn[self.__masteridentifier].commit()
            else:
                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()
        self.__cursor.close()
        self.__None()
        return res

    __startTrans=False
    def startTrans(self):
        "开启事务,仅对 update方法、delete方法、install方法有效"
        self.__startTrans=True
    def commit(self):
        """事务提交

        增删改后的任务进行提交
        """
        if self.__masteridentifier:
            if not self.__config['pattern']:
                self.__conn[self.__masteridentifier].commit()
            else:
                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].commit()

    def rollback(self):
        """事务回滚

        增删改后的任务进行撤销
        """
        if self.__masteridentifier:
            if not self.__config['pattern']:
                self.__conn[self.__masteridentifier].rollback()
            else:
                self.__conn[self.__masteridentifier][self.__dbobjident]['obj'].rollback()
    def getsql(self):
        """得到生成的sql语句"""
        return self.__sql
    __buildSql=None
    def buildSql(self):
        """构造子查询"""
        self.__buildSql=True
        return self
    def __None(self):
        "清除所有赋值条件"
        self.__lock=None
        self.__distinct=None
        self.__join=None
        self.__joinstr=''
        self.__alias=None
        self.__having=None
        self.__group=None
        self.__group1=None
        self.__order=None
        self.__order1=None
        self.__limit=None
        self.__field="*"
        self.__where=None
        self.__wheres=()
        self.__table=None
        self.__buildSql=None
        self.__table=None
        self.__cache=None
    
    __where=None
    __wheres=()
    def where(self,where = None,*wheres):
        """设置过滤条件

        传入方式:
        "id",2 表示id='2'

        "id","in",2,3,4,5,6,...表示 id in (2,3,4,5,6,...)

        "id","in",[2,3,4,5,6,...]表示 id in (2,3,4,5,6,...)


        [("id","gt",6000),"and",("name","like","%超")] 表示 ( id > "6000" and name LIKE "%超" )

        "id","eq",1 表示 id = '1'

        eq 等于
            neq 不等于
            gt 大于
            egt 大于等于
            lt 小于
            elt 小于等于
            like LIKE
        """
        self.__where=where
        self.__wheres=wheres
        return self
    __field='*'
    def field(self,field = "*"):
        """设置过滤显示条件

        参数 field：str 字符串
        """
        self.__field=field
        return self
    __limit=[]
    def limit(self,offset=1, length = None):
        """设置查询数量

        参数 offset：int 起始位置

        参数 length：int 查询数量
        """
        if not offset:
            offset=1
        offset=int(offset)
        if length:
            length=int(length)
        self.__limit=[offset,length]
        return self
    def page(self,pagenow=1, length = 20):
        """设置分页查询

        参数 pagenow：int 页码

        参数 length：int 查询数量
        """
        if not pagenow:
            pagenow=1
        if not length:
            length=20
        pagenow=int(pagenow)
        length=int(length)
        offset=(pagenow-1)*length
        self.__limit=[offset,length]
        return self
    __order=None
    __order1=None
    def order(self,strs=None,*strs1):
        """设置排序查询

        传入方式:

        "id desc"

        "id",'name','appkey','asc'

        "id",'name','appkey'   不包含asc或desc的情况下 默认是desc

        ['id','taskid',{"task_id":"desc"}]
        """
        self.__order=strs
        self.__order1=strs1
        return self
    __group=None
    __group1=None
    def group(self,strs=None,*strs1):
        """设置分组查询

        传入方式:

        "id,name"

        "id","name"
        """
        self.__group=strs
        self.__group1=strs1
        return self
    __having=None
    def having(self,strs=None):
        """用于配合group方法完成从分组的结果中筛选（通常是聚合条件）数据

        参数 strs：string 如："count(time)>3"
        """
        self.__having=strs
        return self
    __alias=None
    def alias(self,strs=None):
        """用于设置当前数据表的别名，便于使用其他的连贯操作例如join方法等。

        参数 strs：string 默认当前表作为别名
        """
        if strs:
            self.__alias=strs
        else:
            self.__alias=self.__table
        return self
    __join=None
    __joinstr=''
    def join(self,strs,on=None,types='INNER'):
        """用于根据两个或多个表中的列之间的关系，从这些表中查询数据

        参数 strs  string 如："test t1"   test表设置别名t1

        参数 on  string 如："t1.id=t2.pid"   设置连接条件

        参数 types  支持INNER、LEFT、RIGHT、FULL  默认INNER

        """
        joinstr=''
        if strs and on:
            joinstr=joinstr+types+" JOIN "+strs+" ON "+on+" "
        if joinstr:
            self.__joinstr=self.__joinstr+joinstr
        return self
    __distinct=None
    def distinct(self,bools=None):
        "用于返回唯一不同的值,配合field方法使用生效,来消除所有重复的记录，并只获取唯一一次记录。"
        self.__distinct=bools
        return self
    __lock=None
    def lock(self,strs=None):
        """用于数据库的锁机制，在查询或者执行操作的时候使用

        排他锁 (FOR UPDATE)

        共享锁 (lock in share mode)
        
        参数 strs  如：True表示自动在生成的SQL语句最后加上FOR UPDATE，

        
        """
        self.__lock=strs
        return self
   
    __cache=[]
    def cache(self,status=None,endtime=86400):
        """设置查询缓存
        status  缓存开关

        参数 endtime：int 缓存时间  0永久
        """
        self.__cache=[status,endtime]
        return self
    def __setsql(self,types=None,data = {}):
        """生成sql语句"""
        if types==None:
            self.__sql="SELECT"
            if self.__distinct and self.__field:
                self.__sql=self.__sql+" DISTINCT"
            if self.__alias:
                self.__sql=self.__sql+" %s FROM %s %s" % (self.__field,self.__table,self.__alias)
            else:
                self.__sql=self.__sql+" %s FROM %s" % (self.__field,self.__table)
        elif types=='count':
            self.__sql="SELECT COUNT(%s) FROM %s" % (self.__field,self.__table)
        elif types=='max':
            self.__sql="SELECT MAX(%s) FROM %s" % (self.__field,self.__table)
        elif types=='min':
            self.__sql="SELECT MIN(%s) FROM %s" % (self.__field,self.__table)
        elif types=='avg':
            self.__sql="SELECT AVG(%s) FROM %s" % (self.__field,self.__table)
        elif types=='sum':
            self.__sql="SELECT SUM(%s) FROM %s" % (self.__field,self.__table)
        elif types=='setinc':
            self.__sql="update %s set %s=%s+%s" % (self.__table,data['field'],data['field'],data['key'])
        elif types=='update':
            strs=''
            for k in data:
                if isinstance(data[k],str):
                    strs=strs+" %s = '%s' ," % (k,escape_string(data[k]))
                else:
                    strs=strs+" %s = %s ," % (k,data[k])
            strs=strs[:-1]
            self.__sql="UPDATE %s SET %s" % (self.__table,strs)
        elif types=='delete':
            self.__sql="DELETE FROM %s" % self.__table
        elif types=='insert':
            if isinstance(data,dict):
                strs=''
                val=''
                for k in data:
                    strs=strs+"%s," % k
                    if isinstance(data[k],str):
                        val=val+"'%s'," % escape_string(data[k])
                    else:
                        val=val+"%s," % data[k]
                strs=strs[:-1]
                val=val[:-1]
                self.__sql="INSERT INTO %s (%s) VALUES (%s)" % (self.__table,strs,val)
            elif isinstance(data,list):
                strs=''
                val='('
                for k in data[0]:
                    strs=strs+" , "+k
                for k in data:
                    for j in k:
                        if isinstance(k[j],str):
                            val=val+"'"+str(k[j])+"',"
                        else:
                            val=val+str(k[j])+","
                    val=val[:-1]
                    val=val+"),("
                val=val[:-2]
                self.__sql="INSERT INTO "+self.__table+" ("+strs[3:]+") VALUES "+val
        if self.__joinstr:
            self.__sql=self.__sql+" "+self.__joinstr
        if self.__where:
            if isinstance(self.__where,str):
                if self.__wheres:
                    if len(self.__wheres) == 2:
                        if isinstance(self.__wheres[1],list):
                            self.__sql=self.__sql + " WHERE %s %s (" % (self.__where,self.__operator(self.__wheres[0]))
                            for k in self.__wheres[1]:
                                self.__sql=self.__sql+str(k)+","
                            self.__sql=self.__sql[:-1]+")"
                        else:
                            self.__sql=self.__sql + " WHERE  %s %s '%s'" % (self.__where,self.__operator(self.__wheres[0]),self.__wheres[1])
                    elif len(self.__wheres) > 2:
                        if self.__wheres[0]=='in':
                            strs=str(self.__wheres[1])
                            i=0
                            for k in self.__wheres:
                                if i > 1:
                                    strs=strs+","+str(k)
                                i=i+1
                            self.__sql=self.__sql + " WHERE  %s in (%s)" % (self.__where,strs)
                    else:
                        self.__sql=self.__sql + " WHERE  %s = '%s'" % (self.__where,self.__wheres[0])
                else:
                    self.__sql=self.__sql + " WHERE  %s" % self.__where
            elif isinstance(self.__where,list):
                self.__sql=self.__sql + " WHERE  %s" % self.__listTrans()
            else:
                print("参数where类型错误")
        if self.__group:
            s=self.__group
            if self.__group1:
                for key in self.__group1:
                    s=s+","+key
            self.__sql=self.__sql+" GROUP BY "+s
        if self.__order:
            s=''
            if isinstance(self.__order,list):
                for strs in self.__order:
                    if isinstance(strs,str):
                        s=s+strs+","
                    else:
                        pass
                        for key in strs:
                            s=s+key+" "+strs[key]
                        s=s+","
                s=s[:-1]
            if isinstance(self.__order,str):
                if self.__order1:
                    if len(self.__order1) > 1:
                        if self.__order1[len(self.__order1)-1] == 'desc' or self.__order1[len(self.__order1)-1] == 'asc':
                            i=0
                            while i<len(self.__order1)-1:
                                s=s+self.__order1[i]+","
                                i=i+1
                            s=s[:-1]+" "+self.__order1[len(self.__order1)-1]
                        else:
                            for key in self.__order1:
                                s=s+key+","
                            s=s[:-1]
                            s=s+" asc"
                        s=self.__order+","+s
                    else:
                        s=s[:-1]+self.__order1[0]
                        s=self.__order+" "+s
                else:
                    s=self.__order
            self.__sql=self.__sql+" ORDER BY "+s
        if self.__having:
            self.__sql=self.__sql+" HAVING "+self.__having
        if self.__limit:
            if self.__limit[1]:
                self.__sql=self.__sql+" LIMIT %d,%d" % (self.__limit[0],self.__limit[1])
            else:
                self.__sql=self.__sql+" LIMIT %d" % self.__limit[0]
        if self.__lock:
            if isinstance(self.__lock,str):
                self.__sql=self.__sql+" "+self.__lock
            else:
                self.__sql=self.__sql+' FOR UPDATE'
    def __listTrans(self):
        """列表转换sql表达式
        返回 字符串
        """
        strs=''
        #[('id', 'eq', '1'), 'or', ('id', 'eq', '2')]
        for k in self.__where:
            if isinstance(k,tuple):
                t=0
                for j in k:
                    if t==0:
                        strs=strs+' '+str(j)+' '
                    elif t==1:
                        strs=strs+self.__operator(j)
                    if t==2:
                        strs=strs+' "'+str(j)+'" '
                    t=t+1
            elif isinstance(k,str):
                strs=strs+k
        return "("+strs+")"
    def __operator(self,strs):
        """运算符转换
        参数 strs 待转的字符串
        返回 已转换的运算符

        符号定义
            eq 等于
            neq 不等于
            gt 大于
            egt 大于等于
            lt 小于
            elt 小于等于
            like LIKE
        """
        strss=strs.upper()
        if strss == 'EQ':
            k='='
        elif strss == 'NEQ':
            k='<>'
        elif strss == 'GT':
            k='>'
        elif strss == 'EGT':
            k='>='
        elif strss == 'LT':
            k='<'
        elif strss == 'ELT':
            k='<='
        elif strss == 'LIKE':
            k='LIKE'
        else:
            k=strss
        return k


# -*- coding: utf-8 -*-
from kcweb.common import globals as kcwglobals
import json,urllib
class args:
    "获取url"
    def params():
        return urllib.parse.parse_qs(urllib.parse.urlparse(kcwglobals.HEADER.URL).query)
    def get(name):
        """获取键值对
        
        name 键

        return 值
        """
        params = args.params()
        try:
            k=params[name][0]
        except:
            k=''
        return k
    def get_json():
        """获取 json get内容

        return  json内容
        """
        try:
            lists=kcwglobals.HEADER.URL.split("?")[1]
            lists=lists.split("&")
            data={}
            for k in lists:
                ar=k.split("=")
                data[ar[0]]=ar[1]
            return data
        except:return None
class froms:
    "获取from  application/x-www-form-urlencoded 的post内容"
    def get(name):
        """获取键值对
        
        name 键

        return 值
        """
        data=kcwglobals.HEADER.BODY_DATA
        params = urllib.parse.parse_qs(urllib.parse.urlparse("?"+str(data)).query)
        try:
            k=urllib.parse.unquote(params[name][0])
        except:
            k=None
        return k
    def get_json():
        """获取post json post内容

        return  json内容
        """
        try:
            lists=kcwglobals.HEADER.BODY_DATA.split("&")
            data={}
            for k in lists:
                ar=k.split("=")
                data[ar[0]]=urllib.parse.unquote(ar[1])
            return data
        except:return None
class binary:
    "二进制或文件处理"
    def get(name):
        """获取文件二进制

        name 文件标识

        return 文件二进制
        """
        return kcwglobals.HEADER.files[name].value
    def filename(name):
        """获取文件名，上传文件时有效，其他情况返回空

        name 文件标识

        return 文件名
        """
        return kcwglobals.HEADER.files[name].filename
    def filesuffix(name):
        """获取文件后缀，上传文件时有效，其他情况返回空

        name 文件标识

        return 文件名
        """
        return kcwglobals.HEADER.files[name].filename.split('.')[-1]
    def save(name,filename):
        """保存二进制文件
        
        name 文件标识

        filename 文件位置

        return 完整文件名
        """
        if kcwglobals.HEADER.files[name].value:
            open(filename, 'wb').write(kcwglobals.HEADER.files[name].value)
            return filename
        else:
            return None
class HEADER:
    def GET():
        return kcwglobals.HEADER.GET
    def Method():
        return kcwglobals.HEADER.Method
    def URL():
        return kcwglobals.HEADER.URL.lstrip()
    def PATH_INFO():
        return kcwglobals.HEADER.PATH_INFO.lstrip()
    def SERVER_PROTOCOL():
        return kcwglobals.HEADER.SERVER_PROTOCOL
    def HTTP_HOST():
        return kcwglobals.HEADER.HTTP_HOST
    def HTTP_COOKIE():
        return kcwglobals.HEADER.HTTP_COOKIE
    def HTTP_USER_AGENT():
        return kcwglobals.HEADER.HTTP_USER_AGENT
    def REMOTE_ADDR():
        return kcwglobals.HEADER.REMOTE_ADDR
    def HTTP_X_REAL_IP():
        "nginx服务器发送的客户端IP"
        return kcwglobals.HEADER.HTTP_X_REAL_IP
    def Physical_IP():
        "获取访问用户物理ip"
        if kcwglobals.HEADER.HTTP_X_REAL_IP:
            return kcwglobals.HEADER.HTTP_X_REAL_IP
        else:
            return kcwglobals.HEADER.REMOTE_ADDR
    # def url_scheme():
    #     "获取http类型  待测试"
    #     return kcwglobals.HEADER.url_scheme
def get_data():
    "获取请求参数体"
    return kcwglobals.HEADER.BODY_DATA
def get_json():
    "获取请求参数体json"
    try:
        return json.loads(kcwglobals.HEADER.BODY_DATA)
    except:
        return None
def getroutecomponent():
    "获取路由"
    return kcwglobals.VAR.component



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import Type
from .isobmff import IsoBmff


class Jpeg(Type):
    """
    Implements the JPEG image type matcher.
    """
    MIME = 'image/jpeg'
    EXTENSION = 'jpg'

    def __init__(self):
        super(Jpeg, self).__init__(
            mime=Jpeg.MIME,
            extension=Jpeg.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 2 and
                buf[0] == 0xFF and
                buf[1] == 0xD8 and
                buf[2] == 0xFF)


class Jpx(Type):
    """
    Implements the JPEG2000 image type matcher.
    """

    MIME = "image/jpx"
    EXTENSION = "jpx"

    def __init__(self):
        super(Jpx, self).__init__(mime=Jpx.MIME, extension=Jpx.EXTENSION)

    def match(self, buf):
        return (
            len(buf) > 50
            and buf[0] == 0x00
            and buf[1] == 0x00
            and buf[2] == 0x00
            and buf[3] == 0x0C
            and buf[16:24] == b"ftypjp2 "
        )


class Png(Type):
    """
    Implements the PNG image type matcher.
    """
    MIME = 'image/png'
    EXTENSION = 'png'

    def __init__(self):
        super(Png, self).__init__(
            mime=Png.MIME,
            extension=Png.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x89 and
                buf[1] == 0x50 and
                buf[2] == 0x4E and
                buf[3] == 0x47)


class Gif(Type):
    """
    Implements the GIF image type matcher.
    """
    MIME = 'image/gif'
    EXTENSION = 'gif'

    def __init__(self):
        super(Gif, self).__init__(
            mime=Gif.MIME,
            extension=Gif.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 2 and
                buf[0] == 0x47 and
                buf[1] == 0x49 and
                buf[2] == 0x46)


class Webp(Type):
    """
    Implements the WEBP image type matcher.
    """
    MIME = 'image/webp'
    EXTENSION = 'webp'

    def __init__(self):
        super(Webp, self).__init__(
            mime=Webp.MIME,
            extension=Webp.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 11 and
                buf[8] == 0x57 and
                buf[9] == 0x45 and
                buf[10] == 0x42 and
                buf[11] == 0x50)


class Cr2(Type):
    """
    Implements the CR2 image type matcher.
    """
    MIME = 'image/x-canon-cr2'
    EXTENSION = 'cr2'

    def __init__(self):
        super(Cr2, self).__init__(
            mime=Cr2.MIME,
            extension=Cr2.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 9 and
                ((buf[0] == 0x49 and buf[1] == 0x49 and
                    buf[2] == 0x2A and buf[3] == 0x0) or
                (buf[0] == 0x4D and buf[1] == 0x4D and
                    buf[2] == 0x0 and buf[3] == 0x2A)) and
                buf[8] == 0x43 and buf[9] == 0x52)


class Tiff(Type):
    """
    Implements the TIFF image type matcher.
    """
    MIME = 'image/tiff'
    EXTENSION = 'tif'

    def __init__(self):
        super(Tiff, self).__init__(
            mime=Tiff.MIME,
            extension=Tiff.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 3 and
                ((buf[0] == 0x49 and buf[1] == 0x49 and
                    buf[2] == 0x2A and buf[3] == 0x0) or
                (buf[0] == 0x4D and buf[1] == 0x4D and
                    buf[2] == 0x0 and buf[3] == 0x2A)))


class Bmp(Type):
    """
    Implements the BMP image type matcher.
    """
    MIME = 'image/bmp'
    EXTENSION = 'bmp'

    def __init__(self):
        super(Bmp, self).__init__(
            mime=Bmp.MIME,
            extension=Bmp.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 1 and
                buf[0] == 0x42 and
                buf[1] == 0x4D)


class Jxr(Type):
    """
    Implements the JXR image type matcher.
    """
    MIME = 'image/vnd.ms-photo'
    EXTENSION = 'jxr'

    def __init__(self):
        super(Jxr, self).__init__(
            mime=Jxr.MIME,
            extension=Jxr.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 2 and
                buf[0] == 0x49 and
                buf[1] == 0x49 and
                buf[2] == 0xBC)


class Psd(Type):
    """
    Implements the PSD image type matcher.
    """
    MIME = 'image/vnd.adobe.photoshop'
    EXTENSION = 'psd'

    def __init__(self):
        super(Psd, self).__init__(
            mime=Psd.MIME,
            extension=Psd.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x38 and
                buf[1] == 0x42 and
                buf[2] == 0x50 and
                buf[3] == 0x53)


class Ico(Type):
    """
    Implements the ICO image type matcher.
    """
    MIME = 'image/x-icon'
    EXTENSION = 'ico'

    def __init__(self):
        super(Ico, self).__init__(
            mime=Ico.MIME,
            extension=Ico.EXTENSION,
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x00 and
                buf[1] == 0x00 and
                buf[2] == 0x01 and
                buf[3] == 0x00)


class Heic(IsoBmff):
    """
    Implements the HEIC image type matcher.
    """
    MIME = 'image/heic'
    EXTENSION = 'heic'

    def __init__(self):
        super(Heic, self).__init__(
            mime=Heic.MIME,
            extension=Heic.EXTENSION
        )

    def match(self, buf):
        if not self._is_isobmff(buf):
            return False

        major_brand, minor_version, compatible_brands = self._get_ftyp(buf)
        if major_brand == 'heic':
            return True
        if major_brand in ['mif1', 'msf1'] and 'heic' in compatible_brands:
            return True
        return False


class Dcm(Type):

    MIME = 'application/dicom'
    EXTENSION = 'dcm'
    OFFSET = 128

    def __init__(self):
        super(Dcm, self).__init__(
            mime=Dcm.MIME,
            extension=Dcm.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > Dcm.OFFSET + 4 and
                buf[Dcm.OFFSET + 0] == 0x44 and
                buf[Dcm.OFFSET + 1] == 0x49 and
                buf[Dcm.OFFSET + 2] == 0x43 and
                buf[Dcm.OFFSET + 3] == 0x4D)



# -*- coding: utf-8 -*-


class Type(object):
    """
    Represents the file type object inherited by
    specific file type matchers.
    Provides convenient accessor and helper methods.
    """
    def __init__(self, mime, extension):
        self.__mime = mime
        self.__extension = extension

    @property
    def mime(self):
        return self.__mime

    @property
    def extension(self):
        return self.__extension

    @property
    def is_extension(self, extension):
        return self.__extension is extension

    @property
    def is_mime(self, mime):
        return self.__mime is mime

    def match(self, buf):
        raise NotImplementedError



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import Type


class Midi(Type):
    """
    Implements the Midi audio type matcher.
    """
    MIME = 'audio/midi'
    EXTENSION = 'midi'

    def __init__(self):
        super(Midi, self).__init__(
            mime=Midi.MIME,
            extension=Midi.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x4D and
                buf[1] == 0x54 and
                buf[2] == 0x68 and
                buf[3] == 0x64)


class Mp3(Type):
    """
    Implements the MP3 audio type matcher.
    """
    MIME = 'audio/mpeg'
    EXTENSION = 'mp3'

    def __init__(self):
        super(Mp3, self).__init__(
            mime=Mp3.MIME,
            extension=Mp3.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 2 and
                ((buf[0] == 0x49 and
                    buf[1] == 0x44 and
                    buf[2] == 0x33) or
                (buf[0] == 0xFF and
                    buf[1] == 0xfb)))


class M4a(Type):
    """
    Implements the M4A audio type matcher.
    """
    MIME = 'audio/m4a'
    EXTENSION = 'm4a'

    def __init__(self):
        super(M4a, self).__init__(
            mime=M4a.MIME,
            extension=M4a.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 10 and
                ((buf[4] == 0x66 and
                    buf[5] == 0x74 and
                    buf[6] == 0x79 and
                    buf[7] == 0x70 and
                    buf[8] == 0x4D and
                    buf[9] == 0x34 and
                    buf[10] == 0x41) or
                (buf[0] == 0x4D and
                    buf[1] == 0x34 and
                    buf[2] == 0x41 and
                    buf[3] == 0x20)))


class Ogg(Type):
    """
    Implements the OGG audio type matcher.
    """
    MIME = 'audio/ogg'
    EXTENSION = 'ogg'

    def __init__(self):
        super(Ogg, self).__init__(
            mime=Ogg.MIME,
            extension=Ogg.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x4F and
                buf[1] == 0x67 and
                buf[2] == 0x67 and
                buf[3] == 0x53)


class Flac(Type):
    """
    Implements the FLAC audio type matcher.
    """
    MIME = 'audio/x-flac'
    EXTENSION = 'flac'

    def __init__(self):
        super(Flac, self).__init__(
            mime=Flac.MIME,
            extension=Flac.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x66 and
                buf[1] == 0x4C and
                buf[2] == 0x61 and
                buf[3] == 0x43)


class Wav(Type):
    """
    Implements the WAV audio type matcher.
    """
    MIME = 'audio/x-wav'
    EXTENSION = 'wav'

    def __init__(self):
        super(Wav, self).__init__(
            mime=Wav.MIME,
            extension=Wav.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 11 and
                buf[0] == 0x52 and
                buf[1] == 0x49 and
                buf[2] == 0x46 and
                buf[3] == 0x46 and
                buf[8] == 0x57 and
                buf[9] == 0x41 and
                buf[10] == 0x56 and
                buf[11] == 0x45)


class Amr(Type):
    """
    Implements the AMR audio type matcher.
    """
    MIME = 'audio/amr'
    EXTENSION = 'amr'

    def __init__(self):
        super(Amr, self).__init__(
            mime=Amr.MIME,
            extension=Amr.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 11 and
                buf[0] == 0x23 and
                buf[1] == 0x21 and
                buf[2] == 0x41 and
                buf[3] == 0x4D and
                buf[4] == 0x52 and
                buf[5] == 0x0A)



# -*- coding: utf-8 -*-
import socket,time,re,os,sys,traceback,threading,urllib,cgi
from . Events import Events
from . common import *
from . import config
from mako.template import Template
from datetime import datetime
from threading import local
from .utill import filetype
from kcweb.utill.cache import cache as kcwcache
from wsgiref.simple_server import make_server
from kcweb.utill.db import mysql as kcwmysql
from kcweb.utill.db import sqlite as kcwsqlite
class web:
    __name=None
    __appname=None
    __config=config 
    def __new__(self,name,appname=None):
        self.__name=name
        self.__appname=appname
        if self.__name != '__main__':
            def apps(env, start_response):
                p=(config.app['staticpath']+env['RAW_URI'].replace(' ',''))
                status='200 ok'
                if os.path.isfile(p):
                    resheader=[("Cache-Control","public, max-age=2592000"),]
                    ETag=md5(p)
                    try:
                        HTTP_IF_NONE_MATCH=env['HTTP_IF_NONE_MATCH']
                    except KeyError:
                        HTTP_IF_NONE_MATCH=None
                    if HTTP_IF_NONE_MATCH:
                        status="304 Not Modified"
                        body=''
                    else:
                        kind = filetype.guess(p)
                        if kind is None:
                            f=open(p,"rb")
                            body=f.read()
                            f.close()
                        else:
                            f=open(p,"rb")
                            body=f.read()
                            f.close()
                            resheader=[
                                ("Content-Type",kind.mime),
                                ("Cache-Control","public, max-age=43200"),
                                ("Accept-Ranges","bytes"),
                            ]
                            resheader.append(('Content-Type',kind.mime))
                            resheader.append(("Accept-Ranges","bytes"))
                    dateArray = datetime.utcfromtimestamp(times()-86400)
                    otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
                    resheader.append(("Last-Modified",otherStyleTime))
                    resheader.append(("ETag",ETag))
                else:
                    files=None
                    try:
                        CONTENT_TYPE=env['CONTENT_TYPE']
                    except:
                        pass
                    else:
                        if 'multipart/form-data; boundary' in CONTENT_TYPE:#上传文件
                            files = cgi.FieldStorage(fp=env['wsgi.input'], environ=env, keep_blank_values=True)
                        else:
                            files=None
                    env['files']=files
                    try:
                        env['BODY_DATA']=str(env['wsgi.input'].next(), encoding = "utf-8")
                    except:
                        env['BODY_DATA']=""
                    try:
                        env['HTTP_X_REAL_IP']=env['HTTP_X_REAL_IP']
                    except:
                        env['HTTP_X_REAL_IP']=""
                    try:
                        env['url_scheme']=env['HTTP_X_REAL_IP']
                    except:
                        env['url_scheme']=env['wsgi.url_scheme'],
                    status,resheader,body=self.__routes(self,env)
                    if type(body) is bytes:
                        pass
                    else:
                        body=bytes(body, encoding='utf-8')
                # print(env)
                start_response(status,resheader)
                return [body]
            return apps
        else:
            return super().__new__(self)
    def run(self,host="127.0.0.1",port="39001",name=None):
        """运行开发环境

        host: 监听地址
        
        port: 端口

        name: python命令行解释机名字
        """
        if 'eventlog' in sys.argv or len(sys.argv)==1:
            filename=sys.argv[0][:-3]
            if self.__config.app['app_debug']:
                arg=sys.argv
                if 'eventlog' in arg:
                    self.__impl(host=host,port=port,filename=filename)
                else:
                    if name:
                        arg.insert(0,name)
                    arg.append('eventlog')
                    Events(arg)
            else:
                self.__impl(
                    host=host,
                    port=port,
                    filename=filename
                )
        elif len(sys.argv)==3 and sys.argv[2]=='--cli':
            try:
                RAW_URI=sys.argv[1]
            except:pass
            else:
                PATH_INFO=RAW_URI.split("?")[0] #/aa/bb/cc
                if PATH_INFO[0]=='/':
                    PATH_INFO=PATH_INFO[1:]
                QUERY_STRING=RAW_URI.replace(str(PATH_INFO),'').replace('?','') #a=1&b=1
                reqheader={ 
                    'REQUEST_METHOD':'GET',
                    'RAW_URI':RAW_URI,
                    'PATH_INFO':PATH_INFO,
                    'QUERY_STRING':QUERY_STRING,
                    'SERVER_PROTOCOL':'',
                    'HTTP_HOST':'',
                    'HTTP_COOKIE':'',
                    'REMOTE_ADDR':'',
                    'HTTP_X_REAL_IP':'',
                    'HTTP_USER_AGENT':'',
                    'BODY_DATA':''
                }
                status,resheader,body=self.__routes(reqheader)
                if 'body' not in body and 'html' not in body and '<' not in body and '>' not in body:
                    print(body)
            exit()
    def __impl(self,host,port,filename):
        "运行测试服务器"
        try:
            if config.app['http_server']=='wsgiref':
                self.__http_wsgi(
                    host=host,
                    port=port,
                    filename=filename
                )
            elif config.app['http_server']=='kcweb':
                self.__http_sever(
                    host=host,
                    port=port,
                    filename=filename
                )
            elif config.app['http_server']=='kcweb_server':
                self.__http_server(
                    host=host,
                    port=port,
                    filename=filename
                )
            else:
                self.__http_wsgi(
                    host=host,
                    port=port,
                    filename=filename
                )
        except KeyboardInterrupt:
            pass
    
    def __getconfigroute(self,PATH_INFO,header):
        "使用配置路由"
        route=self.__config.route
        routedefault=route['default']
        methods=route['methods']
        paths=''
        for path in PATH_INFO:
            paths+="/"+path
        try:
            for item in route['children']:
                if ':' in item['path']:
                    path=item['path'].split(':')
                    if(len(path)==len(PATH_INFO)):
                        is_pp=False
                        try:
                            item['methods']
                        except:pass
                        else:
                            methods=item['methods']
                        for k in methods: #匹配请求方式
                            if header['REQUEST_METHOD'] in k:
                                is_pp=True
                                break
                        if path[0]==paths[:len(path[0])] and is_pp:
                            del PATH_INFO[0]
                            cs=PATH_INFO
                            PATH_INFO=item['component'].split('/')
                            for v in cs:
                                PATH_INFO.append(v)
                            routedefault=True
                            break
                elif item['path']==paths or item['path']+'/'==paths:
                    PATH_INFO=item['component'].split('/')
                    routedefault=True
                    break
        except:pass
        return routedefault,PATH_INFO
    def __get_modular(self,header):
        "获取模块"
        modular=''
        route=self.__config.route
        if route['modular']:
            if isinstance(route['modular'],str):
                modular=route['modular']
            else:
                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
                for mk in route['modular']:
                    if HTTP_HOST in mk:
                        modular=mk[HTTP_HOST]
        return modular
    def __get_plug(self,header):
        "获取插件"
        plug=''
        route=self.__config.route
        if route['plug']:
            if isinstance(route['plug'],str):
                plug=route['plug']
            else:
                HTTP_HOST=header['HTTP_HOST'].split(".")[0]
                for mk in route['plug']:
                    if HTTP_HOST in mk:
                        plug=mk[HTTP_HOST]
        return plug
    def defaultroute(self,header,PATH_INFO):
        "路由匹配"
        route=self.__config.route
        modular=web.__get_modular(self,header)
        if route['plug']:
            getplug=web.__get_plug(self,header)
        else:
            getplug=''
        routedefault=route['default']
        methods=route['methods']
        if routedefault:
            plug=route['defplug']
            files=route['files']
            funct=route['funct']
        else:
            plug=''
            files=''
            funct=''
        param=[]
        urls=''
        i=0
        # HTTP_HOST=header['HTTP_HOST'].split(".")[0]
        ##默认路由start #################################################################################
        
        if modular:
            # if route['plug']:  #匹配模块并且匹配了插件
            #     plug=route['plug']
            if getplug:
                plug=getplug
                routedefault,PATH_INFO=web.__getconfigroute(
                    self,
                    PATH_INFO,
                    header
                )
                if routedefault: #使用路由
                    for path in PATH_INFO:
                        if path:
                            if i==0:
                                files=path
                                urls=urls+"/"+str(path)
                            elif i==1:
                                funct=path
                                urls=urls+"/"+str(path)
                            else:
                                param.append(urllib.parse.unquote(path))
                        i+=1
            else:                  #配置模块没有配置插件
                routedefault,PATH_INFO=web.__getconfigroute(
                    self,
                    PATH_INFO,
                    header
                )
                if routedefault: #使用默认路由
                    for path in PATH_INFO:
                        if path:
                            if i==0:
                                plug=path
                            elif i==1:
                                files=path
                                urls=urls+"/"+str(path)
                            elif i==2:
                                funct=path
                                urls=urls+"/"+str(path)
                            else:
                                param.append(urllib.parse.unquote(path))
                        i+=1
        # elif route['plug']:  #配置模块但没有匹配插件
        #     plug=route['plug']
        elif getplug:
            plug=getplug
            routedefault,PATH_INFO=web.__getconfigroute(
                self,
                PATH_INFO,
                header
            )
            if routedefault: #使用默认路由
                for path in PATH_INFO:
                    if path:
                        if i==0:
                            modular=path
                        elif i==1:
                            files=path
                            urls=urls+"/"+str(path)
                        elif i==2:
                            funct=path
                            urls=urls+"/"+str(path)
                        else:
                            param.append(urllib.parse.unquote(path))
                    i+=1
        else: #完全默认
            routedefault,PATH_INFO=web.__getconfigroute(self,PATH_INFO,header)
            if routedefault: #使用默认路由
                for path in PATH_INFO:
                    if path:
                        if i==0:
                            modular=path
                        elif i==1:
                            plug=path
                        elif i==2:
                            files=path
                            urls=urls+"/"+str(path)
                        elif i==3:
                            funct=path
                            urls=urls+"/"+str(path)
                        else:
                            param.append(urllib.parse.unquote(path))
                    i+=1
        #默认路由end ############################################################
        if not modular:
            modular=route['defmodular']
        
        return methods,modular,plug,files,funct,tuple(param)
    def __tran(self,data,status,resheader):
        "转换控制器返回的内容"
        if isinstance(data,tuple):
            i=0
            for item in data:
                if i==0:
                    body=item
                elif i==1:
                    status=item
                elif i==2:
                    if isinstance(item,dict):
                        for key in item:
                            resheader[key]=item[key]
                    else:
                        raise Exception('错误！这个不是一个字典')
                else:
                    break
                i+=1
        else:
            body=data
        return body,status,resheader
    def __set_globals(self,header):
        globals.HEADER.GET=header
        # globals.HEADER.url_scheme=header['url_scheme']
        globals.HEADER.Method=header['REQUEST_METHOD']
        globals.HEADER.URL=header['RAW_URI']
        globals.HEADER.PATH_INFO=header['PATH_INFO']
        globals.HEADER.QUERY_STRING=header['QUERY_STRING']
        globals.HEADER.SERVER_PROTOCOL=header['SERVER_PROTOCOL']
        globals.HEADER.HTTP_HOST=header['HTTP_HOST']
        globals.HEADER.BODY_DATA=header['BODY_DATA']
        globals.HEADER.REMOTE_ADDR=header['REMOTE_ADDR']
        globals.HEADER.HTTP_X_REAL_IP=header['HTTP_X_REAL_IP']
        try:
            globals.HEADER.files=header['files']
        except:
            globals.HEADER.files=None
        try:
            globals.HEADER.HTTP_COOKIE=header['HTTP_COOKIE']
        except:
            globals.HEADER.HTTP_COOKIE=None
        globals.HEADER.HTTP_USER_AGENT=header['HTTP_USER_AGENT']
    def __del_globals():
        globals.VAR = local()
        globals.HEADER = local()
        globals.G = local()
    def __routes(self,header):
        body="这是一个http测试服务器"
        status="200 ok"
        resheader={"Content-Type":"text/html; charset=utf-8"}
        web.__set_globals(self,header)
        PATH_INFO=header['PATH_INFO'].split('/')
        if PATH_INFO[0]==' ' or PATH_INFO[0]=='':
            del PATH_INFO[0]
        methods,modular,plug,files,funct,param=web.defaultroute(self,header,PATH_INFO)
        globals.VAR.component=(methods,modular,plug,files,funct,param)
        if header['REQUEST_METHOD'] in methods:
            try:
                obj=getattr(web.__appname,modular)
            except (AttributeError,UnboundLocalError):
                status="500 Internal Server Error"
                body=web.__tpl(
                    title = status,
                    e=status,
                    data="路由不存在："+str(modular)+"/"
                )
            else:
                try:
                    obj=getattr(obj,"controller")
                except (AttributeError,UnboundLocalError):
                    status="404 Not Found"
                    body=web.__tpl(
                        title = status,
                        e=status,
                        data="路由不存在："+str(modular)+"/controller/"
                    )
                else:
                    try:
                        obj=getattr(obj,plug)
                    except (AttributeError,UnboundLocalError) as e:
                        con="无法找到目录："+str(modular)+"/controller/"+str(plug)+"/"
                        try:
                            data=getattr(obj,"error")(e,con)
                            body,status,resheader=web.__tran(
                                self,
                                data,
                                status,
                                resheader
                            )
                        except (AttributeError,UnboundLocalError):
                            status="404 Not Found"
                            body=web.__tpl(
                                title = status,
                                e=status,data=con
                            )
                        except Exception as e:
                            status="500 Internal Server Error"
                            errms=status
                            if self.__config.app['app_debug']:
                                print(traceback.format_exc())
                                errms=traceback.format_exc().split("\n")
                            body=web.__tpl(
                                title = status,
                                data=errms,e=e
                            )
                    else:
                        try:
                            obj=getattr(obj,files)
                        except (AttributeError,UnboundLocalError) as e:
                            con="无法找到文件："+str(modular)+"/controller/"+str(plug)+"/"+str(files)+".py"
                            try:
                                data=getattr(obj,"error")(e,con)
                                body,status,resheader=web.__tran(
                                    self
                                    ,data
                                    ,status
                                    ,resheader
                                )
                            except (AttributeError,UnboundLocalError):
                                status="404 Not Found"
                                body=web.__tpl(
                                    title = status
                                    ,data=con
                                    ,e=status)
                            except Exception as e:
                                status="500 Internal Server Error"
                                errms=status
                                if self.__config.app['app_debug']:
                                    print(traceback.format_exc())
                                    errms=traceback.format_exc().split("\n")
                                body=web.__tpl(
                                    title = status,
                                    data=errms,
                                    e=e
                                )
                        else:
                            try:
                                data=None
                                if self.__config.app['before_request']:  #请求前执行的函数
                                    try:
                                        data=getattr(obj,self.__config.app['before_request'])()
                                        if data:
                                            body,status,resheader=web.__tran(
                                                self,data,
                                                status,
                                                resheader
                                            )
                                    except (AttributeError):
                                        print(traceback.format_exc())
                                        pass
                                    except Exception as e:
                                        try:
                                            data=getattr(obj,"error")(e,traceback.format_exc().split("\n"))
                                            body,status,resheader=web.__tran(
                                                self,data,
                                                status,
                                                resheader
                                            )
                                        except (AttributeError):
                                            data=True
                                            status="500 Internal Server Error"
                                            errms=status
                                            if self.__config.app['app_debug']:
                                                # print(traceback.format_exc())
                                                errms=traceback.format_exc().split("\n")
                                            body=web.__tpl(
                                                title = status,
                                                data=errms,e=e
                                            )
                                        except Exception as e:
                                            data=True
                                            status="500 Internal Server Error"
                                            errms=status
                                            if self.__config.app['app_debug']:
                                                print(traceback.format_exc())
                                                errms=traceback.format_exc().split("\n")
                                            body=web.__tpl(
                                                title = status,
                                                data=errms,e=e
                                            )
                                if not data:
                                    data=getattr(obj,funct)(*param)
                                    body,status,resheader=web.__tran(
                                        self,data,
                                        status,
                                        resheader
                                    )
                            except Exception as e:
                                try:
                                    data=getattr(obj,"error")(e,traceback.format_exc().split("\n"))
                                    body,status,resheader=web.__tran(
                                        self,data,
                                        status,
                                        resheader
                                    )
                                except (AttributeError):
                                    status="500 Internal Server Error"
                                    errms=status
                                    if self.__config.app['app_debug']:
                                        print(traceback.format_exc())
                                        errms=traceback.format_exc().split("\n")
                                    body=web.__tpl(
                                        title = status,
                                        data=errms,
                                        e=e
                                    )
                                except Exception as e:
                                    status="500 Internal Server Error"
                                    errms=status
                                    if self.__config.app['app_debug']:
                                        print(traceback.format_exc())
                                        errms=traceback.format_exc().split("\n")
                                    body=web.__tpl(
                                        title = status,
                                        data=errms,
                                        e=e
                                    )
        else:
            status="405 Method Not Allowed"
            body=web.__tpl(
                title = status,
                data='405 Method Not Allowed',
                e=''
            )
        try:
            resheader['set-cookie']=globals.set_cookie
            del globals.set_cookie
        except:pass
        
        if self.__config.app['after_request']:  #请求后执行的函数
            try:
                data=getattr(obj,self.__config.app['after_request'])(body,status,resheader)
                if data:
                    body,status,resheader=web.__tran(self,data,status,resheader)
            except (AttributeError,UnboundLocalError):pass
            except Exception as e:
                try:
                    data=getattr(obj,"error")(e,traceback.format_exc().split("\n"))
                    body,status,resheader=web.__tran(
                        self,data,
                        status,
                        resheader
                    )
                except AttributeError as e:
                    status="500 Internal Server Error"
                    errms=status
                    if self.__config.app['app_debug']:
                        print(traceback.format_exc())
                        errms=traceback.format_exc().split("\n")
                    body=web.__tpl(
                        title = status
                        ,data=errms,
                        e=e
                    )
                except Exception as e:
                    status="500 Internal Server Error"
                    errms=status
                    if self.__config.app['app_debug']:
                        print(traceback.format_exc())
                        errms=traceback.format_exc().split("\n")
                    body=web.__tpl(
                        title = status,
                        data=errms,
                        e=""
                    )
        resheaders=[]
        for key in resheader:
            resheaders.append((key,resheader[key]))
        web.__del_globals()

        #关闭数据库连接
        dbs=kcwmysql.mysql()
        dbs.close()
        # dbs=kcwsqlite.sqlite()
        # dbs.close()


        if isinstance(resheaders,list):
            if not body:
                body=''
            if type(body) is bytes:
                pass
            else:
                body=str(body)
            return str(status),resheaders,body
        else:
            raise Exception()
    def __tpl(**context):
        path=os.path.split(os.path.realpath(__file__))[0]
        body=''
        with open(path+'/tpl/error.html', 'r',encoding='utf-8') as f:
            content=f.read()
            t=Template(content)
            body=t.render(**context)
        return body
    def __application(self,env, start_response):
        # print(env)
        try:
            request_body_size = int(env.get('CONTENT_LENGTH', '0'))
        except (ValueError):
            request_body_size = 0
        if 'multipart/form-data; boundary' in env['CONTENT_TYPE']:#上传文件
            files = cgi.FieldStorage(fp=env['wsgi.input'], environ=env, keep_blank_values=True)
            # open(form["file"].filename, 'wb').write(form["file"].value) 
            # print(files["file"].name)
            # print(files["name"].filename)
            # print(files["name"].value)
            env['BODY_DATA']=None
        else:
            files=None
            BODY_DATA=env['wsgi.input'].read(request_body_size)
            env['BODY_DATA']=BODY_DATA.decode()
        env['RAW_URI']=env['PATH_INFO']
        if env.get('QUERY_STRING'):
            env['RAW_URI']=str(env['PATH_INFO'])+"?"+str(env.get('QUERY_STRING'))
        try:
            env['REMOTE_ADDR']
        except:
            env['REMOTE_ADDR']=''
        try:
            env['wsgi.url_scheme']
        except:
            env['wsgi.url_scheme']=''
        try:
            env['HTTP_X_REAL_IP']
        except:
            env['HTTP_X_REAL_IP']=''
        try:
            env['HTTP_COOKIE']
        except:
            env['HTTP_COOKIE']=''
        try:
            env['HTTP_USER_AGENT']
        except:
            env['HTTP_USER_AGENT']=''
        reqheader={ 
            'REQUEST_METHOD':env['REQUEST_METHOD'],
            'url_scheme':env['wsgi.url_scheme'],
            'RAW_URI':env['RAW_URI'],
            'PATH_INFO':env['PATH_INFO'],
            'QUERY_STRING':env['QUERY_STRING'],
            'SERVER_PROTOCOL':env['SERVER_PROTOCOL'],
            'HTTP_HOST':env['HTTP_HOST'],
            'HTTP_COOKIE':env['HTTP_COOKIE'],
            'REMOTE_ADDR':env['REMOTE_ADDR'],
            'HTTP_USER_AGENT':env['HTTP_USER_AGENT'],
            'BODY_DATA':env['BODY_DATA'],
            'HTTP_X_REAL_IP':env['HTTP_X_REAL_IP'],
            'files':files
        }
        p=(config.app['staticpath']+reqheader['RAW_URI'].replace(' ',''))
        status='200 ok'
        if os.path.isfile(p):
            resheader=[("Cache-Control","public, max-age=2592000"),("Accept-Ranges","bytes")]
            ETag=md5(p)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if HTTP_IF_NONE_MATCH:
                status="304 Not Modified"
            else:
                kind = filetype.guess(p)
                if kind is None:
                    f=open(p,"rb")
                    body=f.read()
                    f.close()
                    if '.css' in p[-5:]:
                        resheader.append(("Content-Type","text/css"))
                    elif '.js' in p[-5:]:
                        resheader.append(("Content-Type","application/javascript"))
                else:
                    f=open(p,"rb")
                    body=f.read()
                    f.close()
                    resheader.append(("Content-Type",kind.mime))
            dateArray = datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            resheader.append(("Last-Modified",otherStyleTime))
            resheader.append(("ETag",ETag))
        else:
            status,resheader,body=web.__routes(self,reqheader)
            resheader.append(('Server', "kcweb"+config.kcweb['version']))
            if type(body) is bytes:
                pass
            else:
                body=bytes(body, encoding='utf-8')
        start_response(status,resheader)
        return [body]
    def __http_wsgi(self,host,port,filename):
        "http——wsgi测试服务"
        print("* \033[1;31;40m! 警告：\033[0m这是一个wsgiref开发服务器。不要在生产环境中部署使用它")
        print('* 生产环境中建议使用gunicorn,gunicorn运行命令如：gunicorn -b '+host+':'+str(port)+' '+str(filename)+':app')
        if self.__config.app['app_debug']:
            print('* 调试器：开启')
        else:
            print('* 调试器：已关闭')
        print("* 运行在http://"+host+":"+str(port)+"/ （按CTRL+C退出）")
        httpd = make_server(host, int(port), self.__application)
        httpd.serve_forever()
    def __http_server(self,host,port,filename):
        "http测试服务"
        tcp_socket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        try:
            tcp_socket.bind((host,int(port)))
        except OSError:
            print("通常每个套接字地址(协议/网络地址/端口)只允许使用一次（按CTRL+C退出）")
        else:
            tcp_socket.listen(1024)
            print('! \033[1;31;40m警告：\033[0m这是一个kcweb_server开发服务器。不要在生产环境中部署使用它')
            print('* 生产环境中建议使用gunicorn,gunicorn运行命令如：gunicorn -b '+host+':'+str(port)+' '+str(filename)+':app')
            if self.__config.app['app_debug']:
                print('* 调试器：开启')
            else:
                print('* 调试器：已关闭')
            print("* 运行在http://"+host+":"+str(port)+"/ （按CTRL+C退出）")
            while True:
                new_tcp_socket,client_info=tcp_socket.accept()
                t=threading.Thread(target=self.__server_client,args=(new_tcp_socket,))
                t.daemon=True
                t.start()
            tcp_socket.close()
    def __server_client(self,new_socket):
        # 处理http的的请求
        data=new_socket.recv(1047576).decode()
        if data:
            datas=data.split("\r\n")
            data1=datas[0]
            #reqsest
            REQUEST_METHOD=data1.split("/")[0].replace(' ','') ##GET
            RAW_URI=re.findall(REQUEST_METHOD+"(.+?) HTTP", data1) #/aa/bb/cc?a=1&b=1
            if RAW_URI:
                RAW_URI=RAW_URI[0]
            else:
                RAW_URI=''
            PATH_INFO=RAW_URI.split("?")[0] #/aa/bb/cc
            QUERY_STRING=RAW_URI.replace(str(PATH_INFO),'').replace('?','') #a=1&b=1
            SERVER_PROTOCOL=data1.split(" ")[-1] #HTTP/1.1
            HTTP_HOST=re.findall("Host: (.+?)\r\n", data)#212.129.149.238:39010
            if HTTP_HOST:
                HTTP_HOST=HTTP_HOST[0]
            else:
                HTTP_HOST=''
            HTTP_COOKIE=re.findall("Cookie: (.+?)\r\n", data)#cookie
            if HTTP_COOKIE:
                HTTP_COOKIE=HTTP_COOKIE[0]
            else:
                HTTP_COOKIE=''
            REMOTE_ADDR=''
            HTTP_USER_AGENT=re.findall("User-Agent: (.+?)\r\n", data) #Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 SE 2.X MetaSr 1.0
            if HTTP_USER_AGENT:
                HTTP_USER_AGENT=HTTP_USER_AGENT[0]
            else:
                HTTP_USER_AGENT=''
            BODY_DATA=datas[len(datas)-1]
            # print(data)
            #reqsest
            reqheader={ 
                'REQUEST_METHOD':REQUEST_METHOD,
                'RAW_URI':RAW_URI,
                'PATH_INFO':PATH_INFO,
                'QUERY_STRING':QUERY_STRING,
                'SERVER_PROTOCOL':SERVER_PROTOCOL,
                'HTTP_HOST':HTTP_HOST,
                'HTTP_COOKIE':HTTP_COOKIE,
                'REMOTE_ADDR':REMOTE_ADDR,
                'HTTP_X_REAL_IP':'',
                'HTTP_USER_AGENT':HTTP_USER_AGENT,
                'BODY_DATA':BODY_DATA
            }
            p=(config.app['staticpath']+RAW_URI.replace(' ',''))
            # print("目录",p)
            status='200 ok'
            if os.path.isfile(p):
                # print('静态文件',p)
                kind = filetype.guess(p)
                if kind is None:
                    
                    f=open(p,"rb")
                    body=f.read()
                    f.close()
                    resheader=[("Cache-Control","public, max-age=43200"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
                    
                    header="HTTP/1.1 %s \n" % status
                    header+="Content-Length:%d\n" % len(body)
                else:
                    f=open(p,"rb")
                    body=f.read()
                    f.close()
                    resheader=[("Content-Type",kind.mime),("Cache-Control","public, max-age=43200"),("Accept-Ranges","bytes"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
                    header="HTTP/1.1 %s \n" % status
                    header+="Content-Length:%d\n" % len(body)
            else:
                status,resheader,body=self.__routes(reqheader)
                if type(body) is bytes:
                    pass
                else:
                    body=body.encode()
                header="HTTP/1.1 %s \n" % status
                header+="Content-Length:%d\n" % len(body)
            
            print(HTTP_HOST+' -- ['+str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))+'] "'+REQUEST_METHOD+" "+RAW_URI +" "+SERVER_PROTOCOL + '" '+status+"-")
            t=time.time()
            header+="Server:kcweb"+str(config.kcweb['version'])+"\n"
            header+="Date:%s\n" % datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
            for t in resheader:
                header+="%s:%s\n" % (t[0],t[1])
            header+="\n"
            try:
                new_socket.send(header.encode())
                new_socket.send(body)
            except Exception as e:
                pass
        new_socket.close() 
    def __http_sever(self,host,port,filename):
        #http测试服务器
        print('\033[1;31;40m! 警告：\033[0m这是一个kcweb开发服务器。不要在生产环境中部署使用它')
        print('* 生产环境中建议使用gunicorn,gunicorn运行命令如：gunicorn -b '+host+':'+str(port)+' '+str(filename)+':app')
        if self.__config.app['app_debug']:
            print('* 调试器：开启')
        else:
            print('* 调试器：已关闭')
        print("* 运行在http://"+host+":"+str(port)+"/ （按CTRL+C退出）")
        tcp_socket=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        tcp_socket.bind((host,int(port)))
        tcp_socket.listen(1024)
        pack_length=1024
        tcp_socket.setblocking(False)
        tcp_socket_list=list()
        while True:
            try:
                new_tcp_socket,client_info=tcp_socket.accept()
            except:
                pass
            else:
                new_tcp_socket.setblocking(False)
                tcp_socket_list.append(new_tcp_socket)
            for cli_soc in tcp_socket_list:
                try:
                    data=cli_soc.recv(pack_length).decode()
                except Exception as e:
                    pass
                else:
                    if data:
                        datas=data.split("\r\n")
                        data1=datas[0]
                        #reqsest
                        REQUEST_METHOD=data1.split("/")[0].replace(' ','') ##GET
                        RAW_URI=re.findall(REQUEST_METHOD+"(.+?) HTTP", data1) #/aa/bb/cc?a=1&b=1
                        if RAW_URI:
                            RAW_URI=RAW_URI[0]
                        else:
                            RAW_URI=''
                        PATH_INFO=RAW_URI.split("?")[0] #/aa/bb/cc
                        QUERY_STRING=RAW_URI.replace(str(PATH_INFO),'').replace('?','') #a=1&b=1
                        SERVER_PROTOCOL=data1.split(" ")[-1] #HTTP/1.1
                        HTTP_HOST=re.findall("Host: (.+?)\r\n", data)#212.129.149.238:39010
                        if HTTP_HOST:
                            HTTP_HOST=HTTP_HOST[0]
                        else:
                            HTTP_HOST=''
                        HTTP_COOKIE=re.findall("Cookie: (.+?)\r\n", data)#cookie
                        if HTTP_COOKIE:
                            HTTP_COOKIE=HTTP_COOKIE[0]
                        else:
                            HTTP_COOKIE=''
                        REMOTE_ADDR=''
                        HTTP_USER_AGENT=re.findall("User-Agent: (.+?)\r\n", data) #Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36 SE 2.X MetaSr 1.0
                        if HTTP_USER_AGENT:
                            HTTP_USER_AGENT=HTTP_USER_AGENT[0]
                        else:
                            HTTP_USER_AGENT=''
                        BODY_DATA=datas[len(datas)-1]
                        # print(data)
                        #reqsest
                        reqheader={ 
                            'REQUEST_METHOD':REQUEST_METHOD,
                            'RAW_URI':RAW_URI,
                            'PATH_INFO':PATH_INFO,
                            'QUERY_STRING':QUERY_STRING,
                            'SERVER_PROTOCOL':SERVER_PROTOCOL,
                            'HTTP_HOST':HTTP_HOST,
                            'HTTP_COOKIE':HTTP_COOKIE,
                            'REMOTE_ADDR':REMOTE_ADDR,
                            'HTTP_X_REAL_IP':'',
                            'HTTP_USER_AGENT':HTTP_USER_AGENT,
                            'BODY_DATA':BODY_DATA
                        }
                        p=(config.app['staticpath']+RAW_URI.replace(' ',''))
                        # print("目录",p)
                        status='200 ok'
                        if os.path.isfile(p):
                            # print('静态文件',p)
                            kind = filetype.guess(p)
                            if kind is None:
                                
                                f=open(p,"rb")
                                body=f.read()
                                f.close()
                                resheader=[("Cache-Control","public, max-age=43200"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
                                
                                header="HTTP/1.1 %s \n" % status
                                header+="Content-Length:%d\n" % len(body)
                            else:
                                f=open(p,"rb")
                                body=f.read()
                                f.close()
                                resheader=[("Content-Type",kind.mime),("Cache-Control","public, max-age=43200"),("Accept-Ranges","bytes"),("Expires","Thu, 07 Nov 2019 02:59:02 GMT")]
                                header="HTTP/1.1 %s \n" % status
                                header+="Content-Length:%d\n" % len(body)
                        else:
                            status,resheader,body=self.__routes(reqheader)
                            if type(body) is bytes:
                                pass
                            else:
                                body=body.encode()
                            header="HTTP/1.1 %s \n" % status
                            header+="Content-Length:%d\n" % len(body)
                        
                        print(HTTP_HOST+' -- ['+str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))+'] "'+REQUEST_METHOD+" "+RAW_URI +" "+SERVER_PROTOCOL + '" '+status+"-")
                        t=time.time()
                        header+="Server:kcweb"+str(config.kcweb['version'])+"\n"
                        header+="Date:%s\n" % datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
                        for t in resheader:
                            header+="%s:%s\n" % (t[0],t[1])
                        header+="\n"
                        try:
                            cli_soc.send(header.encode())
                            cli_soc.send(body)
                        except Exception as e:
                            cli_soc.close()
                    else:
                        cli_soc.close()
                        tcp_socket_list.remove(cli_soc)
        tcp_socket.close()



# -*- coding: utf-8 -*-
import requests,traceback
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
class Http:
    "http请求类"
    set_proxies=None  #设置代理
    set_cookies={} #设置请求cookie
    set_header={'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36'} #请求头
    set_timeout=10 #超时 20秒
    set_max_retries=2 #重试次数 (实际请求3次)
    set_verify=False  #SSL 证书的验证 sll证书路径
    set_encoding="utf-8" #设置text输出编码
    set_session=True #是否启用会话

    get_header={} #获取响应头
    get_cookies={} #获取最后的响应cookie
    get_cookie_str='' #获取最后的响应cookie 字符串
    get_text='' #获取body响应内容
    get_content='' #获取body响应二进制内容
    get_response='' #获取响应对象
    get_status_code=None #获取响应状态码
    
    req=None
    def gettext(self):
        """得到响应text"""
        return self.get_text
    def openurl(self,url,method="GET",data=None,params=None,files=None,allow_redirects=True):
        """模拟浏览器请求

        url : 目标地址

        method ：GET POST 等

        data：请求参数

        file 上传文件

        allow_redirects 是否重定向
        """
        if self.set_session:
            if self.req is None:
                self.req = requests.Session()
                self.req.mount('http://', requests.adapters.HTTPAdapter(max_retries=self.set_max_retries))
                self.req.mount('https://', requests.adapters.HTTPAdapter(max_retries=self.set_max_retries))
        else:
            if self.req is None:
                self.req = requests
        if self.set_cookies and isinstance(self.set_cookies,str):
            self.cookieserTdict()
        response=self.req.request(method, url,data=data,params=params,files=files,proxies=self.set_proxies,cookies=self.set_cookies,headers=self.set_header,timeout=self.set_timeout,verify=self.set_verify,allow_redirects=allow_redirects)
        response.encoding=self.set_encoding
        self.get_header=dict(response.headers)
        cookie=requests.utils.dict_from_cookiejar(response.cookies)
        if self.get_cookies and cookie:
            self.get_cookies=self.__merge(self.get_cookies,cookie)
        elif cookie:
            self.get_cookies=cookie
        if self.set_cookies:
            self.get_cookies=self.__merge(self.set_cookies,self.get_cookies)
        if self.get_cookies:
            cookies=''
            for key in self.get_cookies:
                cookies=cookies+key+"="+self.get_cookies[key]+";"
            self.get_cookie_str=cookies
        self.get_text=response.text
        self.get_content=response.content
        self.get_response=response
        self.get_status_code=int(response.status_code)
    def __merge(self,dict1, dict2):
        "合并两个字典"
        C_dict = {}
        for key,value in dict1.items():
            C_dict[key]=value
        for key,value in dict2.items():
            C_dict[key]=value
        return C_dict
    def cookieserTdict(self):
        "cookies字符串转换字典"
        if isinstance(self.set_cookies,str):
            cok={}
            for line in self.set_cookies.split(";"):
                lists=line.split("=",1)
                # print(lists[])
                if lists[0]:
                    cok[lists[0]]=lists[1]
            self.set_cookies=cok


# -*- coding: utf-8 -*-

_NUM_SIGNATURE_BYTES = 262


def get_signature_bytes(path):
    """
    Reads file from disk and returns the first 262 bytes
    of data representing the magic number header signature.

    Args:
        path: path string to file.

    Returns:
        First 262 bytes of the file content as bytearray type.
    """
    with open(path, 'rb') as fp:
        return bytearray(fp.read(_NUM_SIGNATURE_BYTES))


def signature(array):
    """
    Returns the first 262 bytes of the given bytearray
    as part of the file header signature.

    Args:
        array: bytearray to extract the header signature.

    Returns:
        First 262 bytes of the file content as bytearray type.
    """
    length = len(array)
    index = _NUM_SIGNATURE_BYTES if length > _NUM_SIGNATURE_BYTES else length

    return array[:index]


def get_bytes(obj):
    """
    Infers the input type and reads the first 262 bytes,
    returning a sliced bytearray.

    Args:
        obj: path to readable, file, bytes or bytearray.

    Returns:
        First 262 bytes of the file content as bytearray type.

    Raises:
        TypeError: if obj is not a supported type.
    """
    try:
        obj = obj.read(_NUM_SIGNATURE_BYTES)
    except AttributeError:
        # duck-typing as readable failed - we'll try the other options
        pass

    kind = type(obj)

    if kind is bytearray:
        return signature(obj)

    if kind is str:
        return get_signature_bytes(obj)

    if kind is bytes:
        return signature(obj)

    if kind is memoryview:
        return signature(obj).tolist()

    raise TypeError('Unsupported type as file input: %s' % kind)



# -*- coding: utf-8 -*- 
import os, sys, time, subprocess,signal
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
eventgtimexz=0
class MyFileSystemEventHander(FileSystemEventHandler):
    def __init__(self, fn):
        super(MyFileSystemEventHander, self).__init__()
        self.restart = fn

    def on_any_event(self, event):
        if '.py' in event.src_path and event.src_path.endswith('.py'):
            global eventgtimexz
            if time.time()-eventgtimexz > 3:
                eventgtimexz=time.time()
                # print('* 更新文件：%s' % event.src_path)
                # time.sleep(10)
                time.sleep(0.2)
                self.restart()
class Events:
    command = ['echo', 'ok']
    process = None
    def __init__(self,argv):
        if '--server' not in argv and 'python' not in argv[0] or 'kcw.py' in argv:
            argv.insert(0, 'python')
        self.command = argv
        paths = os.path.abspath('.')
        self.start_watch(paths, None)

    def kill_process(self):
        "关闭"
        if self.process:
            self.process.kill()
            self.process.wait()
            self.process = None

    def start_process(self):
        "启动"
        self.process = subprocess.Popen(self.command, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr)

    def restart_process(self):
        "重启"
        self.kill_process()
        time.sleep(0.1)
        self.start_process()

    def start_watch(self,path, callback):
        "执行"
        observer = Observer()
        observer.schedule(MyFileSystemEventHander(self.restart_process), path, recursive=True)
        observer.start()
        self.start_process()
        try:
            while True:
                time.sleep(0.1)
        except KeyboardInterrupt:
            self.kill_process()
            # observer.stop()
        # observer.join()
    
# Events(['server.py'])  #执行server.py文件



# python3 setup.py sdist upload
# twine upload --repository-url https://test.pypi.org/legacy/ dist/* #上传到测试
# pip install --index-url https://pypi.org/simple/ kcweb   #安装测试服务上的kcweb pip3 install kcweb==4.12.4 -i https://pypi.org/simple/
############################################# 
from setuptools import setup, find_packages,Extension
import os
def file_get_content(k):
    "获取文件内容"
    if os.path.isfile(k):
        f=open(k,'r',encoding="utf-8")
        con=f.read()
        f.close()
    else:
        con=''
    return con
confkcw={}
confkcw['name']='kcweb'                             #项目的名称 
confkcw['version']='4.13.2'							#项目版本
confkcw['description']=''       #项目的简单描述
confkcw['long_description']="增加任务队列进度条"     #项目详细描述
confkcw['license']='MIT License'                    #开源协议   mit开源
confkcw['url']=''
confkcw['author']='禄可集团-坤坤'  					 #名字
confkcw['author_email']='fk1402936534@qq.com' 	     #邮件地址
confkcw['maintainer']='坤坤' 						 #维护人员的名字
confkcw['maintainer_email']='fk1402936534@qq.com'    #维护人员的邮件地址
def get_file(folder='./',lists=[]):
    lis=os.listdir(folder)
    for files in lis:
        if not os.path.isfile(folder+"/"+files):
            if files=='__pycache__' or files=='.git':
                pass
            else:
                lists.append(folder+"/"+files)
                get_file(folder+"/"+files,lists)
        else:
            pass
    return lists
b=get_file("kcweb",['kcweb'])
setup(
    name = confkcw["name"],
    version = confkcw["version"],
    keywords = "kcweb"+confkcw['version'],
    description = confkcw["description"],
    long_description = confkcw["long_description"],
    license = confkcw["license"],
    author = confkcw["author"],
    author_email = confkcw["author_email"],
    maintainer = confkcw["maintainer"],
    maintainer_email = confkcw["maintainer_email"],
    url=confkcw['url'],
    packages =  b,
    # data_files=[('Scripts', ['kcweb/bin/kcw.exe'])],
    install_requires = ['pymongo>=3.10.0','Mako>=1.1.2','requests>=2.23.0','six>=1.12.0','watchdog>=0.9.0','websockets==8.1'], #第三方包
    package_data = {
        '': ['*.html', '*.js','*.css','*.jpg','*.png','*.gif'],
    }
)


from queue import Queue
from .db import model
from .db import sqlite as kcwsqlite
import threading,time,os,hashlib,random
queuesdbpath=os.path.split(os.path.realpath(__file__))[0]+"/Queues"
class model_kcweb_task(model.model):
    "任务"
    config={'type':'sqlite','db':queuesdbpath}
    model.dbtype.conf=config
    table="model_kcweb_queues" 
    fields={
        "id":model.dbtype.int(LEN=11,PRI=True,A_L=True),        #设置id为自增主键
        "taskid":model.dbtype.varchar(LEN=32,DEFAULT=''),        #设置id为自增主键
        "title":model.dbtype.varchar(LEN=1024,DEFAULT=''),      #名称
        "describes":model.dbtype.varchar(LEN=2048,DEFAULT=''),  #描述
        "code":model.dbtype.int(LEN=11,DEFAULT=2),              #状态码 0成功 1失败 2等待中 3正在执行  4完成
        "msg":model.dbtype.text(),                              #状态描述
        "error":model.dbtype.text(),                            #异常信息
        "start":model.dbtype.int(LEN=11,DEFAULT=0),             #进度条起始值
        "end":model.dbtype.int(LEN=11,DEFAULT=100),               #进度条结束值
        "starts":model.dbtype.varchar(LEN=11,DEFAULT=0),             #每秒钟进度条起始值增加多少
        "addtime":model.dbtype.int(LEN=11,DEFAULT=0),            #添加时间
        "endtime":model.dbtype.int(LEN=11,DEFAULT=0),            #结束时间
        "updtime":model.dbtype.int(LEN=11,DEFAULT=0)            #更新时间
    }
class Queues():
    __globalqueue=None
    def start():
        if not Queues.__globalqueue:
            if not os.path.isfile(queuesdbpath):
                model_kcweb_tasks=model_kcweb_task()
                model_kcweb_tasks.create_table()
            try:
                kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").find()
            except:
                model_kcweb_tasks=model_kcweb_task()
                model_kcweb_tasks.create_table()
            Queues.__globalqueue=Queue()
            t=threading.Thread(target=Queues.__messagequeue)
            t.daemon=True
            t.start()
    def __messagequeue():
        # time.sleep(0.01)
        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code","in",2,3).delete()
        while True:
            if not Queues.__globalqueue.empty():
                value=Queues.__globalqueue.get()
                kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":3,"msg":"正在执行","error":""})
                if value['args']:
                    try:
                        value['target'](*value['args'])
                    except Exception as e:
                        import traceback
                        print(traceback.print_exc())
                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"失败","error":str(e)})
                    else:
                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"执行完成"})
                else:
                    try:
                        value['target']()
                    except Exception as e:
                        import traceback
                        print(traceback.print_exc())
                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":1,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"失败","error":str(e)})
                    else:
                        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid = '"+value['task']['taskid']+"' and code!=4").update({"code":4,"start":100,'endtime':int(time.time()),'updtime':int(time.time()),"msg":"执行完成"})
            else:
                time.sleep(1)
    def delhist():
        """清除任务历史记录(包括 成功的 失败的 已完成的)"""
        return kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code","in",0,1,4).delete()
    def setstart(taskid,start=1):
        """设置进度条位置
        
        start 支持1到99的整数
        """
        arr=kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid",taskid).find()
        if arr:
            start=int(start)
            if start>=1 and start <=99:
                dqsjc=int(time.time())-arr['updtime'] #当前时间差
                if dqsjc>=1:
                    starts=round(float((start-arr['start'])/dqsjc),2)
                    kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid",taskid).update({"start":start,'starts':starts,'updtime':int(time.time())})
                return True
            else:
                return False
        else:
            return False
    def insert(target,args=None,title="默认任务",describes="",msg='等待中',taskid=None,start=0,updtime=0): #add_queue
        """添加队列
        
        target 方法名  必须

        args 方法参数 非必须  如 (参数1,参数2)

        title 任务名称

        describes 任务描述

        msg 状态描述

        taskid 任务id

        start 进度条开始位置 （建议1到50）

        updtime 进度条更新时间

        return taskid
        """
        Queues.start()
        ttt=int(time.time())
        if not taskid:
            m = hashlib.md5()
            m.update((str(ttt)+str(random.randint(100000,999999))).encode(encoding='utf-8'))
            taskid=m.hexdigest()
        end=100
        if not updtime:
            updtime=ttt
        task={"taskid":taskid,"title":title,"describes":describes,"code":2,"msg":msg,"error":"","start":start,"end":end,"addtime":ttt,'starts':0,'endtime':0,'updtime':updtime}
        key={"target":target,"args":args,"task":task}
        kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").insert(task)
        Queues.__globalqueue.put(key)
        return taskid
    def getall(pagenow=1,pagesize=20,where=None):
        """获取全部队列

        code 1获取失败的任务   2获取等待中的任务   3获取正在执行中的任务  4获取执行完成的任务

        return list
        """
        if not os.path.isfile(queuesdbpath):
            model_kcweb_tasks=model_kcweb_task()
            model_kcweb_tasks.create_table()
        try:
            lists=kcwsqlite.sqlite().connect(queuesdbpath).where(where).table("model_kcweb_queues").order("id desc").page(pagenow,pagesize).select()
            count=kcwsqlite.sqlite().connect(queuesdbpath).where(where).table("model_kcweb_queues").count()
            return lists,count
        except:
            model_kcweb_tasks=model_kcweb_task()
            model_kcweb_tasks.create_table()
            return [],0
    def status(taskid):
        """获取任务状态
        
        taskid  任务id

        return dict
        """
        arr=kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("taskid",taskid).find()
        if arr:
            dqsjc=arr['updtime']-arr['addtime']
            if dqsjc and arr['start']>=1:
                arr['starts']=round(arr['start']/dqsjc,2)
            else:
                arr['starts']=round(arr['starts'],2)
        return arr
    def iscomplete():
        """判断对列中的任务是否全部执行完成
        
        return Boolean
        """
        if kcwsqlite.sqlite().connect(queuesdbpath).table("model_kcweb_queues").where("code","in",2,3).count():
            return True
        else:
            return False



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import Type
from .isobmff import IsoBmff


class Mp4(IsoBmff):
    """
    Implements the MP4 video type matcher.
    """
    MIME = 'video/mp4'
    EXTENSION = 'mp4'

    def __init__(self):
        super(Mp4, self).__init__(
            mime=Mp4.MIME,
            extension=Mp4.EXTENSION
        )

    def match(self, buf):
        if not self._is_isobmff(buf):
            return False

        major_brand, minor_version, compatible_brands = self._get_ftyp(buf)
        return major_brand in ['mp41', 'mp42']


class M4v(Type):
    """
    Implements the M4V video type matcher.
    """
    MIME = 'video/x-m4v'
    EXTENSION = 'm4v'

    def __init__(self):
        super(M4v, self).__init__(
            mime=M4v.MIME,
            extension=M4v.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 10 and
                buf[0] == 0x0 and buf[1] == 0x0 and
                buf[2] == 0x0 and buf[3] == 0x1C and
                buf[4] == 0x66 and buf[5] == 0x74 and
                buf[6] == 0x79 and buf[7] == 0x70 and
                buf[8] == 0x4D and buf[9] == 0x34 and
                buf[10] == 0x56)


class Mkv(Type):
    """
    Implements the MKV video type matcher.
    """
    MIME = 'video/x-matroska'
    EXTENSION = 'mkv'

    def __init__(self):
        super(Mkv, self).__init__(
            mime=Mkv.MIME,
            extension=Mkv.EXTENSION
        )

    def match(self, buf):
        return ((len(buf) > 15 and
                buf[0] == 0x1A and buf[1] == 0x45 and
                buf[2] == 0xDF and buf[3] == 0xA3 and
                buf[4] == 0x93 and buf[5] == 0x42 and
                buf[6] == 0x82 and buf[7] == 0x88 and
                buf[8] == 0x6D and buf[9] == 0x61 and
                buf[10] == 0x74 and buf[11] == 0x72 and
                buf[12] == 0x6F and buf[13] == 0x73 and
                buf[14] == 0x6B and buf[15] == 0x61) or
                (len(buf) > 38 and
                    buf[31] == 0x6D and buf[32] == 0x61 and
                    buf[33] == 0x74 and buf[34] == 0x72 and
                    buf[35] == 0x6f and buf[36] == 0x73 and
                    buf[37] == 0x6B and buf[38] == 0x61))


class Webm(Type):
    """
    Implements the WebM video type matcher.
    """
    MIME = 'video/webm'
    EXTENSION = 'webm'

    def __init__(self):
        super(Webm, self).__init__(
            mime=Webm.MIME,
            extension=Webm.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x1A and
                buf[1] == 0x45 and
                buf[2] == 0xDF and
                buf[3] == 0xA3)


class Mov(IsoBmff):
    """
    Implements the MOV video type matcher.
    """
    MIME = 'video/quicktime'
    EXTENSION = 'mov'

    def __init__(self):
        super(Mov, self).__init__(
            mime=Mov.MIME,
            extension=Mov.EXTENSION
        )

    def match(self, buf):
        if not self._is_isobmff(buf):
            return False

        major_brand, minor_version, compatible_brands = self._get_ftyp(buf)
        return major_brand == 'qt  '


class Avi(Type):
    """
    Implements the AVI video type matcher.
    """
    MIME = 'video/x-msvideo'
    EXTENSION = 'avi'

    def __init__(self):
        super(Avi, self).__init__(
            mime=Avi.MIME,
            extension=Avi.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 10 and
                buf[0] == 0x52 and
                buf[1] == 0x49 and
                buf[2] == 0x46 and
                buf[3] == 0x46 and
                buf[8] == 0x41 and
                buf[9] == 0x56 and
                buf[10] == 0x49)


class Wmv(Type):
    """
    Implements the WMV video type matcher.
    """
    MIME = 'video/x-ms-wmv'
    EXTENSION = 'wmv'

    def __init__(self):
        super(Wmv, self).__init__(
            mime=Wmv.MIME,
            extension=Wmv.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 9 and
                buf[0] == 0x30 and
                buf[1] == 0x26 and
                buf[2] == 0xB2 and
                buf[3] == 0x75 and
                buf[4] == 0x8E and
                buf[5] == 0x66 and
                buf[6] == 0xCF and
                buf[7] == 0x11 and
                buf[8] == 0xA6 and
                buf[9] == 0xD9)


class Flv(Type):
    """
    Implements the FLV video type matcher.
    """
    MIME = 'video/x-flv'
    EXTENSION = 'flv'

    def __init__(self):
        super(Flv, self).__init__(
            mime=Flv.MIME,
            extension=Flv.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x46 and
                buf[1] == 0x4C and
                buf[2] == 0x56 and
                buf[3] == 0x01)


class Mpeg(Type):
    """
    Implements the MPEG video type matcher.
    """
    MIME = 'video/mpeg'
    EXTENSION = 'mpg'

    def __init__(self):
        super(Mpeg, self).__init__(
            mime=Mpeg.MIME,
            extension=Mpeg.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x0 and
                buf[1] == 0x0 and
                buf[2] == 0x1 and
                buf[3] >= 0xb0 and
                buf[3] <= 0xbf)



# -*- coding: utf-8 -*-
from kcweb.utill import rediss as red
from kcweb import config
import json
class redis:
    "redis  注意：连接池链接模式下不支持动态配置"
    db=config.redis['db'] #默认数据库
    __redisObj=None
    __coninfo={}
    __config=config.redis
    __identifier=''
    def __connects(self):
        """设置redis链接"""
        if self.__config['pattern']:
            self.__identifier=self.__config['host']+self.__config['password']+str(self.__config['port'])+str(self.__config['db'])
            if self.__identifier not in self.__coninfo:
                if self.__config['password']:
                    redis_pool=red.ConnectionPool(host=self.__config['host'],password=self.__config['password'],port=self.__config['port'],db=self.__config['db'])
                else:
                    redis_pool=red.ConnectionPool(host=self.__config['host'],port=self.__config['port'],db=self.__config['db'])
                self.__coninfo[self.__identifier]=red.Redis(connection_pool=redis_pool)
                if config.app['app_debug']:
                    print("建立redis连接池",self.__identifier)
            self.__redisObj=self.__coninfo[self.__identifier]
            self.__config['db']=self.db
        else:
            if self.__config['password']:
                self.__redisObj=red.Redis(host=self.__config['host'],password=self.__config['password'],port=self.__config['port'],db=self.__config['db'])
            else:
                self.__redisObj=red.Redis(host=self.__config['host'],port=self.__config['port'],db=self.__config['db'])
            if config.app['app_debug']:
                print("建立redis连接",self.__identifier)
        
    def __json_decode(self,strs):
        """json字符串转python类型"""
        try:
            return json.loads(strs)
        except Exception:
            return {}
    def __json_encode(self,strs):
        """转成字符串"""
        try:
            return json.dumps(strs,ensure_ascii=False)
        except Exception:
            return ""
    def getconfig(self):
        return self.__config
    def connect(self,config):
        """设置redis链接信息 

        参数 config 参考配置信息格式

        返回 redis
        """ 
        if config:
            if isinstance(config,int):
                self.__config['db']=config
            elif isinstance(config,dict):
                if "host" in config:
                    self.__config['host']=config['host']
                if "port" in config:
                    self.__config['port']=config['port']
                if "password" in config:
                    self.__config['password']=config['password']
                if "db" in config:
                    self.__config['db']=config['db']
            else:
                raise Exception("配置信息错误")
        return self
    def redisObj(self):
        "得到一个redis连接对象，执行更多高级操作"
        self.__connects()
        return self.__redisObj
    def getstr(self,name):
        """获取name的值

        name，键
        返回键“name”处的值，如果该键不存在，则返回“none”
        """
        self.__connects()
        return self.__redisObj.get(name)
    def setstr(self,name,value,ex=None, px=None, nx=False, xx=False):
        """
        name，键

        value，值 只能是字符串

        ex，过期时间（秒）

        px，过期时间（毫秒）

        nx，如果设置为True，则只有key不存在时，当前set操作才执行,同#setnx(key, value)

        xx，如果设置为True，则只有key存在时，当前set操作才执行
        """
        if not self.__redisObj:
            self.__connects()
        if not ex and not px:
            if self.__config['ex']:
                ex=self.__config['ex']
        return self.__redisObj.set(name, value, ex=ex, px=px, nx=nx, xx=xx)
    def append(self,name,value):
        """将字符串“value”追加到“name”处的值。如果``键`` 不存在，请使用值“name”创建它。 返回位于“name”的值的新长度。
        
        name，键

        value，值 只能是字符串
        """
        self.__connects()
        return self.__redisObj.append(name,value)
    
    def set(self,name,value,ex=None, px=None, nx=False, xx=False):
        """
        name，键

        value，值 可以是字典 列表 或字符串

        ex，过期时间（秒）

        px，过期时间（毫秒）

        nx，如果设置为True，则只有key不存在时，当前set操作才执行

        xx，如果设置为True，则只有key存在时，当前set操作才执行
        """
        self.__connects()
        if not ex and not px:
            if self.__config['ex']:
                ex=self.__config['ex']
        value=self.__json_encode(value)
        return self.__redisObj.set(name, value, ex=ex, px=px, nx=nx, xx=xx)
    
    def get(self,name):
        """获取name的值

        name，键
        返回键“name”处的值，如果该键不存在，则返回“none”
        """
        self.__connects()
        value=self.__redisObj.get(name)
        if value:
            value=self.__json_decode(value)
        return value
    def delete(self,name):
        """删除name的值

        name，键
        
        返回 True，如果该键不存在，则返回 0
        """
        self.__connects()
        return self.__redisObj.delete(name)
    def rpush(self,name, *values):
        "元素从list的右边加入 ，可以添加多个"
        self.__connects()
        # print(self.__config)
        return self.__redisObj.rpush(name, *values) 
    def rpop(self,name):
        "元素从list的右边移出"
        self.__connects()
        return self.__redisObj.rpop(name)
    def rpoplpush(self,src, dst):
        "元素从list的右边移出,并且从list的左边加入"
        self.__connects()
        return self.__redisObj.rpoplpush(src, dst)
    def rpushx(self,name, value):
        "当name存在时，元素才能从list的右边加入"
        self.__connects()
        return self.__redisObj.rpushx(name, value)
    def lpush(self,name, *values):
        "元素从list的左边加入，可以添加多个"
        self.__connects()
        return self.__redisObj.lpush(name, *values)
    def lpop(self,name):
        "元素从list的左边移出"
        self.__connects()
        return self.__redisObj.lpop(name)
    def lpushxs(self,name):
        "当name存在时，元素才能从list的左边加入"
        self.__connects()
        return self.__redisObj.lpushx(name)
    def hset(self,name,key,value):
        """在hash名称中将key设置为value如果HSET创建了新字段，则返回1，否则返回0
        
        name，名

        key，键

        mapping，值
        """
        self.__connects()
        return self.__redisObj.hset(name,key,value)
    def hget(self,name,key):
        "返回hash的name中的key值"
        self.__connects()
        return self.__redisObj.hget(name,key)
    def hgetall(self,name):
        "返回hash名称/值对的Python dict"
        self.__connects()
        return self.__redisObj.hgetall(name)
    def hmset(self,name,mapping,ex=None):
        """在hash的name中为每个键设置值
        name，键

        mapping，值

        ex,过期时间(秒)
        
        """
        self.__connects()
        boot = self.__redisObj.hmset(name,mapping)

        if not ex:
            if self.__config['ex']:
                ex=self.__config['ex']
        if ex:
            self.__redisObj.expire(name,ex)
        return boot
    def hmget(self,name, keys, *args):
        "返回与“keys”顺序相同的值列表``"
        self.__connects()
        return self.__redisObj.hmget(name, keys, *args)
    
